@@ -1,21 +1,21 @@
 
 /*
- * $Id: acl.c,v 1.205.2.13 2000/01/05 23:40:00 wessels Exp $
+ * $Id: acl.c,v 1.225.2.6 2001/01/28 16:24:45 hno Exp $
  *
  * DEBUG: section 28    Access Control
  * AUTHOR: Duane Wessels
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  Duane Wessels and the University of California San Diego.  Please
- *  see the COPYRIGHT file for full details.  Squid incorporates
- *  software developed and/or copyrighted by other sources.  Please see
- *  the CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -120,6 +120,9 @@ strtokFile(void)
 		debug(28, 0) ("strtokFile: %s not found\n", fn);
 		return (NULL);
 	    }
+#if defined(_SQUID_CYGWIN_)
+	    setmode(fileno(aclFile), O_TEXT);
+#endif
 	    aclFromFile = 1;
 	} else {
 	    return t;
@@ -182,6 +185,8 @@ aclStrToType(const char *s)
 #if USE_IDENT
     if (!strcmp(s, "ident"))
 	return ACL_IDENT;
+    if (!strcmp(s, "ident_regex"))
+	return ACL_IDENT_REGEX;
 #endif
     if (!strncmp(s, "proto", 5))
 	return ACL_PROTO;
@@ -191,6 +196,8 @@ aclStrToType(const char *s)
 	return ACL_BROWSER;
     if (!strcmp(s, "proxy_auth"))
 	return ACL_PROXY_AUTH;
+    if (!strcmp(s, "proxy_auth_regex"))
+	return ACL_PROXY_AUTH_REGEX;
     if (!strcmp(s, "src_as"))
 	return ACL_SRC_ASN;
     if (!strcmp(s, "dst_as"))
@@ -205,6 +212,8 @@ aclStrToType(const char *s)
     if (!strcmp(s, "arp"))
 	return ACL_SRC_ARP;
 #endif
+    if (!strcmp(s, "req_mime_type"))
+	return ACL_REQ_MIME_TYPE;
     return ACL_NONE;
 }
 
@@ -240,6 +249,8 @@ aclTypeToStr(squid_acl type)
 #if USE_IDENT
     if (type == ACL_IDENT)
 	return "ident";
+    if (type == ACL_IDENT_REGEX)
+	return "ident_regex";
 #endif
     if (type == ACL_PROTO)
 	return "proto";
@@ -249,6 +260,8 @@ aclTypeToStr(squid_acl type)
 	return "browser";
     if (type == ACL_PROXY_AUTH)
 	return "proxy_auth";
+    if (type == ACL_PROXY_AUTH_REGEX)
+	return "proxy_auth_regex";
     if (type == ACL_SRC_ASN)
 	return "src_as";
     if (type == ACL_DST_ASN)
@@ -263,6 +276,8 @@ aclTypeToStr(squid_acl type)
     if (type == ACL_SRC_ARP)
 	return "arp";
 #endif
+    if (type == ACL_REQ_MIME_TYPE)
+	return "req_mime_type";
     return "ERROR";
 }
 
@@ -336,8 +351,6 @@ aclParseMethodList(void *curlist)
     while ((t = strtokFile())) {
 	q = memAllocate(MEM_INTLIST);
 	q->i = (int) urlParseMethod(t);
-	if (q->i == METHOD_PURGE)
-	    Config.onoff.enable_purge = 1;
 	*(Tail) = q;
 	Tail = &q->next;
     }
@@ -391,9 +404,9 @@ decode_addr(const char *asc, struct in_addr *addr, struct in_addr *mask)
 
 
 #define SCAN_ACL1       "%[0123456789.]-%[0123456789.]/%[0123456789.]"
-#define SCAN_ACL2       "%[0123456789.]-%[0123456789.]"
+#define SCAN_ACL2       "%[0123456789.]-%[0123456789.]%c"
 #define SCAN_ACL3       "%[0123456789.]/%[0123456789.]"
-#define SCAN_ACL4       "%[0123456789.]"
+#define SCAN_ACL4       "%[0123456789.]%c"
 
 static acl_ip_data *
 aclParseIpData(const char *t)
@@ -406,6 +419,7 @@ aclParseIpData(const char *t)
     acl_ip_data **Q;
     struct hostent *hp;
     char **x;
+    char c;
     debug(28, 5) ("aclParseIpData: %s\n", t);
     if (!strcasecmp(t, "all")) {
 	q->addr1.s_addr = 0;
@@ -415,11 +429,11 @@ aclParseIpData(const char *t)
     }
     if (sscanf(t, SCAN_ACL1, addr1, addr2, mask) == 3) {
 	(void) 0;
-    } else if (sscanf(t, SCAN_ACL2, addr1, addr2) == 2) {
+    } else if (sscanf(t, SCAN_ACL2, addr1, addr2, &c) == 2) {
 	mask[0] = '\0';
     } else if (sscanf(t, SCAN_ACL3, addr1, mask) == 2) {
 	addr2[0] = '\0';
-    } else if (sscanf(t, SCAN_ACL4, addr1) == 1) {
+    } else if (sscanf(t, SCAN_ACL4, addr1, &c) == 1) {
 	addr2[0] = '\0';
 	mask[0] = '\0';
     } else if (sscanf(t, "%[^/]/%s", addr1, mask) == 2) {
@@ -474,6 +488,9 @@ aclParseIpData(const char *t)
 	safe_free(q);
 	return NULL;
     }
+    if ((q->addr1.s_addr & q->mask.s_addr) != q->addr1.s_addr ||
+	(q->addr2.s_addr & q->mask.s_addr) != q->addr2.s_addr)
+	debug(28, 0) ("aclParseIpData: WARNING: Netmask masks away part of the specified IP in '%s'\n", t);
     q->addr1.s_addr &= q->mask.s_addr;
     q->addr2.s_addr &= q->mask.s_addr;
     /* 1.2.3.4/255.255.255.0  --> 1.2.3.0 */
@@ -554,7 +571,7 @@ aclParseTimeSpec(void *curlist)
 		debug(28, 0) ("%s line %d: %s\n",
 		    cfg_filename, config_lineno, config_input_line);
 		debug(28, 0) ("aclParseTimeSpec: IGNORING Bad time range\n");
-		xfree(q);
+		memFree(q, MEM_ACL_TIME_DATA);
 		return;
 	    }
 	    q->start = h1 * 60 + m1;
@@ -563,7 +580,7 @@ aclParseTimeSpec(void *curlist)
 		debug(28, 0) ("%s line %d: %s\n",
 		    cfg_filename, config_lineno, config_input_line);
 		debug(28, 0) ("aclParseTimeSpec: IGNORING Reversed time range\n");
-		xfree(q);
+		memFree(q, MEM_ACL_TIME_DATA);
 		return;
 	    }
 	}
@@ -720,6 +737,9 @@ aclParseAclLine(acl ** head)
     case ACL_IDENT:
 	aclParseWordList(&A->data);
 	break;
+    case ACL_IDENT_REGEX:
+	aclParseRegexList(&A->data);
+	break;
 #endif
     case ACL_PROTO:
 	aclParseProtoList(&A->data);
@@ -735,6 +755,14 @@ aclParseAclLine(acl ** head)
 	    assert(proxy_auth_cache);
 	}
 	break;
+    case ACL_PROXY_AUTH_REGEX:
+	aclParseRegexList(&A->data);
+	if (!proxy_auth_cache) {
+	    /* First time around, 7921 should be big enough */
+	    proxy_auth_cache = hash_create((HASHCMP *) strcmp, 7921, hash_string);
+	    assert(proxy_auth_cache);
+	}
+	break;
 #if SQUID_SNMP
     case ACL_SNMP_COMMUNITY:
 	aclParseWordList(&A->data);
@@ -745,6 +773,9 @@ aclParseAclLine(acl ** head)
 	aclParseArpList(&A->data);
 	break;
 #endif
+    case ACL_REQ_MIME_TYPE:
+	aclParseRegexList(&A->data);
+	break;
     case ACL_NONE:
     default:
 	fatal("Bad ACL type");
@@ -759,7 +790,7 @@ aclParseAclLine(acl ** head)
     if (A->data == NULL) {
 	debug(28, 0) ("aclParseAclLine: IGNORING invalid ACL: %s\n",
 	    A->cfgline);
-	xfree(A);
+	memFree(A, MEM_ACL);
 	return;
     }
     /* append */
@@ -829,14 +860,14 @@ aclParseDenyInfoLine(acl_deny_info_list ** head)
 	debug(28, 0) ("aclParseDenyInfoLine: missing 'error page' parameter.\n");
 	return;
     }
-    A = xcalloc(1, sizeof(acl_deny_info_list));
+    A = memAllocate(MEM_ACL_DENY_INFO_LIST);
     A->err_page_id = errorReservePageId(t);
     A->err_page_name = xstrdup(t);
     A->next = (acl_deny_info_list *) NULL;
     /* next expect a list of ACL names */
     Tail = &A->acl_list;
     while ((t = strtok(NULL, w_space))) {
-	L = xcalloc(1, sizeof(acl_name_list));
+	L = memAllocate(MEM_ACL_NAME_LIST);
 	xstrncpy(L->name, t, ACL_NAME_SZ);
 	*Tail = L;
 	Tail = &L->next;
@@ -845,7 +876,7 @@ aclParseDenyInfoLine(acl_deny_info_list ** head)
 	debug(28, 0) ("%s line %d: %s\n",
 	    cfg_filename, config_lineno, config_input_line);
 	debug(28, 0) ("aclParseDenyInfoLine: deny_info line contains no ACL's, skipping\n");
-	xfree(A);
+	memFree(A, MEM_ACL_DENY_INFO_LIST);
 	return;
     }
     for (B = *head, T = head; B; T = &B->next, B = B->next);	/* find the tail */
@@ -880,7 +911,7 @@ aclParseAccessLine(acl_access ** head)
 	debug(28, 0) ("%s line %d: %s\n",
 	    cfg_filename, config_lineno, config_input_line);
 	debug(28, 0) ("aclParseAccessLine: expecting 'allow' or 'deny', got '%s'.\n", t);
-	xfree(A);
+	memFree(A, MEM_ACL_ACCESS);
 	return;
     }
 
@@ -901,7 +932,7 @@ aclParseAccessLine(acl_access ** head)
 	    debug(28, 0) ("%s line %d: %s\n",
 		cfg_filename, config_lineno, config_input_line);
 	    debug(28, 0) ("aclParseAccessLine: ACL name '%s' not found.\n", t);
-	    xfree(L);
+	    memFree(L, MEM_ACL_LIST);
 	    continue;
 	}
 	L->acl = a;
@@ -912,7 +943,7 @@ aclParseAccessLine(acl_access ** head)
 	debug(28, 0) ("%s line %d: %s\n",
 	    cfg_filename, config_lineno, config_input_line);
 	debug(28, 0) ("aclParseAccessLine: Access line contains no ACL's, skipping\n");
-	xfree(A);
+	memFree(A, MEM_ACL_ACCESS);
 	return;
     }
     A->cfgline = xstrdup(config_input_line);
@@ -1027,6 +1058,9 @@ aclDecodeProxyAuth(const char *proxy_auth, char **user, char **password, char *b
     debug(28, 6) ("aclDecodeProxyAuth: cleartext = '%s'\n", cleartext);
     xstrncpy(buf, cleartext, bufsize);
     xfree(cleartext);
+    /* Trim leading whitespace after decoding */
+    while (xisspace(*buf))
+	buf++;
     *user = buf;
     if ((*password = strchr(*user, ':')) != NULL)
 	*(*password)++ = '\0';
@@ -1051,7 +1085,7 @@ aclDecodeProxyAuth(const char *proxy_auth, char **user, char **password, char *b
  */
 
 static int
-aclMatchProxyAuth(wordlist * data, const char *proxy_auth, acl_proxy_auth_user * auth_user, aclCheck_t * checklist)
+aclMatchProxyAuth(void *data, const char *proxy_auth, acl_proxy_auth_user * auth_user, aclCheck_t * checklist, squid_acl acltype)
 {
     /* checklist is used to register user name when identified, nothing else */
     LOCAL_ARRAY(char, login_buf, USER_IDENT_SZ);
@@ -1095,7 +1129,7 @@ aclMatchProxyAuth(wordlist * data, const char *proxy_auth, acl_proxy_auth_user *
 	    auth_user->expiretime = current_time.tv_sec + Config.authenticateTTL;
 	    auth_user->ip_expiretime = squid_curtime + Config.authenticateIpTTL;
 	    auth_user->ipaddr = checklist->src_addr;
-	    hash_join(proxy_auth_cache, (hash_link *) auth_user);
+	    hash_join(proxy_auth_cache, &auth_user->hash);
 	    /* Continue checking below, as normal */
 	}
     }
@@ -1118,15 +1152,31 @@ aclMatchProxyAuth(wordlist * data, const char *proxy_auth, acl_proxy_auth_user *
 	    auth_user->ipaddr = checklist->src_addr;
 	    /* copy username to request for logging on client-side */
 	    xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
-	    return aclMatchUser(data, user);
+	    switch (acltype) {
+	    case ACL_PROXY_AUTH:
+		return aclMatchUser(data, user);
+	    case ACL_PROXY_AUTH_REGEX:
+		return aclMatchRegex(data, user);
+	    default:
+		fatal("aclMatchProxyAuth: unknown ACL type");
+		return 0;	/* NOTREACHED */
+	    }
 	} else {
-	    /* user has switched to another IP addr */
-	    debug(28, 1) ("aclMatchProxyAuth: user '%s' has changed IP address\n", user);
-	    /* remove this user from the hash, making him unknown */
-	    hash_remove_link(proxy_auth_cache, (hash_link *) auth_user);
-	    aclFreeProxyAuthUser(auth_user);
-	    /* require the user to reauthenticate */
-	    return -2;
+	    if (Config.onoff.authenticateIpTTLStrict) {
+		/* Access from some other IP address than the one owning
+		 * this user ID. Deny access
+		 */
+		debug(28, 1) ("aclMatchProxyAuth: user '%s' tries to use multple IP addresses!\n", user);
+		return 0;
+	    } else {
+		/* user has switched to another IP addr */
+		debug(28, 1) ("aclMatchProxyAuth: user '%s' has changed IP address\n", user);
+		/* remove this user from the hash, making him unknown */
+		hash_remove_link(proxy_auth_cache, (hash_link *) auth_user);
+		aclFreeProxyAuthUser(auth_user);
+		/* require the user to reauthenticate */
+		return -2;
+	    }
 	}
     } else {
 	/* password mismatch/timeout */
@@ -1171,7 +1221,7 @@ aclLookupProxyAuthStart(aclCheck_t * checklist)
     debug(28, 4) ("aclLookupProxyAuthStart: going to ask authenticator on %s\n", user);
     /* we must still check this user's password */
     auth_user = memAllocate(MEM_ACL_PROXY_AUTH_USER);
-    auth_user->user = xstrdup(user);
+    auth_user->hash.key = xstrdup(user);
     auth_user->passwd = xstrdup(password);
     auth_user->passwd_ok = -1;
     auth_user->expiretime = -1;
@@ -1278,6 +1328,25 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
     int k;
     if (!ae)
 	return 0;
+    switch (ae->type) {
+    case ACL_DST_IP:
+    case ACL_DST_DOMAIN:
+    case ACL_DST_DOM_REGEX:
+    case ACL_URLPATH_REGEX:
+    case ACL_URL_PORT:
+    case ACL_PROTO:
+    case ACL_METHOD:
+    case ACL_DST_ASN:
+	/* These ACL types require checklist->request */
+	if (NULL == r) {
+	    debug(28, 1) ("WARNING: '%s' ACL is used but there is no"
+		" HTTP request -- access denied.\n", ae->name);
+	    return 0;
+	}
+	break;
+    default:
+	break;
+    }
     debug(28, 3) ("aclMatchAcl: checking '%s'\n", ae->cfgline);
     switch (ae->type) {
     case ACL_SRC_IP:
@@ -1374,7 +1443,7 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	/* NOTREACHED */
     case ACL_MAXCONN:
 	k = clientdbEstablished(checklist->src_addr, 0);
-	return ((k > ((intlist *) ae->data)->i) ? 0 : 1);
+	return ((k > ((intlist *) ae->data)->i) ? 1 : 0);
 	/* NOTREACHED */
     case ACL_URL_PORT:
 	return aclMatchIntegerRange(ae->data, (int) r->port);
@@ -1391,6 +1460,14 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	    return 0;
 	}
 	/* NOTREACHED */
+    case ACL_IDENT_REGEX:
+	if (checklist->ident[0]) {
+	    return aclMatchRegex(ae->data, checklist->ident);
+	} else {
+	    checklist->state[ACL_IDENT] = ACL_LOOKUP_NEEDED;
+	    return 0;
+	}
+	/* NOTREACHED */
 #endif
     case ACL_PROTO:
 	return aclMatchInteger(ae->data, r->protocol);
@@ -1405,6 +1482,7 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	return aclMatchRegex(ae->data, browser);
 	/* NOTREACHED */
     case ACL_PROXY_AUTH:
+    case ACL_PROXY_AUTH_REGEX:
 	if (NULL == r) {
 	    return -1;
 	} else if (!r->flags.accelerated) {
@@ -1434,7 +1512,8 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	switch (aclMatchProxyAuth(ae->data,
 		header,
 		checklist->auth_user,
-		checklist)) {
+		checklist,
+		ae->type)) {
 	case 0:
 	    /* Correct password, but was not allowed in this ACL */
 	    return 0;
@@ -1483,6 +1562,13 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
     case ACL_SRC_ARP:
 	return aclMatchArp(&ae->data, checklist->src_addr);
 #endif
+    case ACL_REQ_MIME_TYPE:
+	header = httpHeaderGetStr(&checklist->request->header,
+	    HDR_CONTENT_TYPE);
+	if (NULL == header)
+	    header = "";
+	return aclMatchRegex(ae->data, header);
+	/* NOTREACHED */
     case ACL_NONE:
     default:
 	debug(28, 0) ("aclMatchAcl: '%s' has bad type %d\n",
@@ -1628,12 +1714,6 @@ aclCheck(aclCheck_t * checklist)
 void
 aclChecklistFree(aclCheck_t * checklist)
 {
-    if (checklist->state[ACL_SRC_DOMAIN] == ACL_LOOKUP_PENDING)
-	fqdncacheUnregister(checklist->src_addr, checklist);
-    if (checklist->state[ACL_DST_DOMAIN] == ACL_LOOKUP_PENDING)
-	fqdncacheUnregister(checklist->dst_addr, checklist);
-    if (checklist->state[ACL_DST_IP] == ACL_LOOKUP_PENDING)
-	ipcacheUnregister(checklist->request->host, checklist);
     if (checklist->request)
 	requestUnlink(checklist->request);
     checklist->request = NULL;
@@ -1800,7 +1880,7 @@ static void
 aclFreeProxyAuthUser(void *data)
 {
     acl_proxy_auth_user *u = data;
-    xfree(u->user);
+    xfree(u->hash.key);
     xfree(u->passwd);
     memFree(u, MEM_ACL_PROXY_AUTH_USER);
 }
@@ -1842,6 +1922,10 @@ aclDestroyAcls(acl ** head)
 	case ACL_TIME:
 	    aclDestroyTimeList(a->data);
 	    break;
+#if USE_IDENT
+	case ACL_IDENT_REGEX:
+#endif
+	case ACL_PROXY_AUTH_REGEX:
 	case ACL_URL_REGEX:
 	case ACL_URLPATH_REGEX:
 	case ACL_BROWSER:
@@ -1916,7 +2000,7 @@ aclDestroyDenyInfoList(acl_deny_info_list ** list)
 	}
 	a_next = a->next;
 	xfree(a->err_page_name);
-	safe_free(a);
+	memFree(a, MEM_ACL_DENY_INFO_LIST);
     }
     *list = NULL;
 }
@@ -1940,37 +2024,23 @@ aclDestroyIntRange(intrange * list)
 static int
 aclDomainCompare(const void *a, const void *b)
 {
-    const char *d1 = a;
-    const char *d2 = b;
-    int l1;
-    int l2;
-    while ('.' == *d1)
-	d1++;
-    while ('.' == *d2)
-	d2++;
-    l1 = strlen(d1);
-    l2 = strlen(d2);
-    while (d1[--l1] == d2[--l2]) {
-	if ((l1 == 0) && (l2 == 0))
-	    return 0;		/* d1 == d2 */
-	if (0 == l1) {
-	    if ('.' == d2[l2 - 1]) {
-		debug(28, 0) ("WARNING: %s is a subdomain of %s\n", d2, d1);
-		debug(28, 0) ("WARNING: This may break Splay tree searching\n");
-		debug(28, 0) ("WARNING: You should remove '%s' from the ACL named '%s'\n", d2, AclMatchedName);
-	    }
-	    return -1;		/* d1 < d2 */
-	}
-	if (0 == l2) {
-	    if ('.' == d1[l1 - 1]) {
-		debug(28, 0) ("WARNING: %s is a subdomain of %s\n", d1, d2);
-		debug(28, 0) ("WARNING: This may break Splay tree searching\n");
-		debug(28, 0) ("WARNING: You should remove '%s' from the ACL named '%s'\n", d1, AclMatchedName);
-	    }
-	    return 1;		/* d1 > d2 */
-	}
+    const char *d1;
+    const char *d2;
+    int ret;
+    d1 = b;
+    d2 = a;
+    ret = aclHostDomainCompare(d1, d2);
+    if (ret != 0) {
+	d1 = a;
+	d2 = b;
+	ret = aclHostDomainCompare(d1, d2);
     }
-    return (d1[l1] - d2[l2]);
+    if (ret == 0) {
+	debug(28, 0) ("WARNING: '%s' is a subdomain of '%s'\n", d1, d2);
+	debug(28, 0) ("WARNING: because of this '%s' is ignored to keep splay tree searching predictable\n", a);
+	debug(28, 0) ("WARNING: You should probably remove '%s' from the ACL named '%s'\n", d1, AclMatchedName);
+    }
+    return ret;
 }
 
 /* compare a host and a domain */
@@ -2167,6 +2237,11 @@ aclDumpGeneric(const acl * a)
 #endif
 #if USE_IDENT
     case ACL_IDENT:
+	return wordlistDup(a->data);
+	break;
+    case ACL_IDENT_REGEX:
+	return aclDumpRegexList(a->data);
+	break;
 #endif
     case ACL_PROXY_AUTH:
 	return wordlistDup(a->data);
@@ -2174,6 +2249,7 @@ aclDumpGeneric(const acl * a)
     case ACL_TIME:
 	return aclDumpTimeSpecList(a->data);
 	break;
+    case ACL_PROXY_AUTH_REGEX:
     case ACL_URL_REGEX:
     case ACL_URLPATH_REGEX:
     case ACL_BROWSER:
@@ -2209,7 +2285,26 @@ aclDumpGeneric(const acl * a)
     return NULL;
 }
 
-
+/*
+ * This function traverses all ACL elements referenced
+ * by an access list (presumably 'http_access').   If 
+ * it finds a PURGE method ACL, then it returns TRUE,
+ * otherwise FALSE.
+ */
+int
+aclPurgeMethodInUse(acl_access * a)
+{
+    acl_list *b;
+    for (; a; a = a->next) {
+	for (b = a->acl_list; b; b = b->next) {
+	    if (ACL_METHOD != b->acl->type)
+		continue;
+	    if (aclMatchInteger(b->acl->data, METHOD_PURGE))
+		return 1;
+	}
+    }
+    return 0;
+}
 
 
 #if USE_ARP_ACL
@@ -2236,9 +2331,14 @@ aclDumpGeneric(const acl * a)
  *
  * NOTE: Linux code by David Luyer <luyer@ucs.uwa.edu.au>.
  *       Original (BSD-specific) code no longer works.
+ *       Solaris code by R. Gancarz <radekg@solaris.elektrownia-lagisza.com.pl>
  */
 
+#ifdef _SQUID_SOLARIS_
+#include <sys/sockio.h>
+#else
 #include <sys/sysctl.h>
+#endif
 #ifdef _SQUID_LINUX_
 #include <net/if_arp.h>
 #include <sys/ioctl.h>
@@ -2247,7 +2347,9 @@ aclDumpGeneric(const acl * a)
 #endif
 #include <net/route.h>
 #include <net/if.h>
+#if HAVE_NETINET_IF_ETHER_H
 #include <netinet/if_ether.h>
+#endif
 
 /*
  * Decode an ascii representation (asc) of an ethernet adress, and place
@@ -2311,10 +2413,10 @@ aclParseArpList(void *curlist)
 /***************/
 /* aclMatchArp */
 /***************/
-#ifdef _SQUID_LINUX_
 static int
 aclMatchArp(void *dataptr, struct in_addr c)
 {
+#if defined(_SQUID_LINUX_)
     struct arpreq arpReq;
     struct sockaddr_in ipAddr;
     unsigned char ifbuffer[sizeof(struct ifreq) * 64];
@@ -2340,7 +2442,7 @@ aclMatchArp(void *dataptr, struct in_addr c)
     ipAddr.sin_port = 0;
     ipAddr.sin_addr = c;
     memset(&arpReq, '\0', sizeof(arpReq));
-    memcpy(&arpReq.arp_pa, &ipAddr, sizeof(struct sockaddr_in));
+    xmemcpy(&arpReq.arp_pa, &ipAddr, sizeof(struct sockaddr_in));
     /* Query ARP table */
     if (ioctl(HttpSockets[0], SIOCGARP, &arpReq) != -1) {
 	/* Skip non-ethernet interfaces */
@@ -2386,7 +2488,7 @@ aclMatchArp(void *dataptr, struct in_addr c)
 	ipAddr.sin_port = 0;
 	ipAddr.sin_addr = c;
 	memset(&arpReq, '\0', sizeof(arpReq));
-	memcpy(&arpReq.arp_pa, &ipAddr, sizeof(struct sockaddr_in));
+	xmemcpy(&arpReq.arp_pa, &ipAddr, sizeof(struct sockaddr_in));
 	strncpy(arpReq.arp_dev, ifr->ifr_name, sizeof(arpReq.arp_dev) - 1);
 	arpReq.arp_dev[sizeof(arpReq.arp_dev) - 1] = '\0';
 	/* Query ARP table */
@@ -2428,6 +2530,48 @@ aclMatchArp(void *dataptr, struct in_addr c)
 	 * exist on multiple interfaces?
 	 */
     }
+#elif defined(_SQUID_SOLARIS_)
+    struct arpreq arpReq;
+    struct sockaddr_in ipAddr;
+    unsigned char ifbuffer[sizeof(struct ifreq) * 64];
+    struct ifconf ifc;
+    struct ifreq *ifr;
+    int offset;
+    splayNode **Top = dataptr;
+    /*
+     * Set up structures for ARP lookup with blank interface name
+     */
+    ipAddr.sin_family = AF_INET;
+    ipAddr.sin_port = 0;
+    ipAddr.sin_addr = c;
+    memset(&arpReq, '\0', sizeof(arpReq));
+    xmemcpy(&arpReq.arp_pa, &ipAddr, sizeof(struct sockaddr_in));
+    /* Query ARP table */
+    if (ioctl(HttpSockets[0], SIOCGARP, &arpReq) != -1) {
+	/*
+	 *  Solaris (at least 2.6/x86) does not use arp_ha.sa_family -
+	 * it returns 00:00:00:00:00:00 for non-ethernet media 
+	 */
+	if (arpReq.arp_ha.sa_data[0] == 0 &&
+	    arpReq.arp_ha.sa_data[1] == 0 &&
+	    arpReq.arp_ha.sa_data[2] == 0 &&
+	    arpReq.arp_ha.sa_data[3] == 0 &&
+	    arpReq.arp_ha.sa_data[4] == 0 &&
+	    arpReq.arp_ha.sa_data[5] == 0)
+	    return 0;
+	debug(28, 4) ("Got address %02x:%02x:%02x:%02x:%02x:%02x\n",
+	    arpReq.arp_ha.sa_data[0] & 0xff, arpReq.arp_ha.sa_data[1] & 0xff,
+	    arpReq.arp_ha.sa_data[2] & 0xff, arpReq.arp_ha.sa_data[3] & 0xff,
+	    arpReq.arp_ha.sa_data[4] & 0xff, arpReq.arp_ha.sa_data[5] & 0xff);
+	/* Do lookup */
+	*Top = splay_splay(&arpReq.arp_ha.sa_data, *Top, aclArpCompare);
+	debug(28, 3) ("aclMatchArp: '%s' %s\n",
+	    inet_ntoa(c), splayLastResult ? "NOT found" : "found");
+	return (0 == splayLastResult);
+    }
+#else
+    WRITE ME;
+#endif
     /*
      * Address was not found on any interface
      */
@@ -2438,6 +2582,7 @@ aclMatchArp(void *dataptr, struct in_addr c)
 static int
 aclArpCompare(const void *a, const void *b)
 {
+#if defined(_SQUID_LINUX_)
     const unsigned short *d1 = a;
     const unsigned short *d2 = b;
     if (d1[0] != d2[0])
@@ -2446,22 +2591,28 @@ aclArpCompare(const void *a, const void *b)
 	return (d1[1] > d2[1]) ? 1 : -1;
     if (d1[2] != d2[2])
 	return (d1[2] > d2[2]) ? 1 : -1;
+#elif defined(_SQUID_SOLARIS_)
+    const unsigned char *d1 = a;
+    const unsigned char *d2 = b;
+    if (d1[0] != d2[0])
+	return (d1[0] > d2[0]) ? 1 : -1;
+    if (d1[1] != d2[1])
+	return (d1[1] > d2[1]) ? 1 : -1;
+    if (d1[2] != d2[2])
+	return (d1[2] > d2[2]) ? 1 : -1;
+    if (d1[3] != d2[3])
+	return (d1[3] > d2[3]) ? 1 : -1;
+    if (d1[4] != d2[4])
+	return (d1[4] > d2[4]) ? 1 : -1;
+    if (d1[5] != d2[5])
+	return (d1[5] > d2[5]) ? 1 : -1;
+#else
+    WRITE ME;
+#endif
     return 0;
 }
-#else
-
-static int
-aclMatchArp(void *dataptr, struct in_addr c)
-{
-    WRITE ME;
-}
-
-static int
-aclArpCompare(const void *data, splayNode * n)
-{
-    WRITE ME;
-}
 
+#if UNUSED_CODE
 /**********************************************************************
 * This is from the pre-splay-tree code for BSD
 * I suspect the Linux approach will work on most O/S and be much
