@@ -1,21 +1,21 @@
 
 /*
- * $Id: ftp.c,v 1.286.2.14 1999/12/22 21:05:30 wessels Exp $
+ * $Id: ftp.c,v 1.298.2.4 2001/01/12 00:51:47 wessels Exp $
  *
  * DEBUG: section 9     File Transfer Protocol (FTP)
  * AUTHOR: Harvest Derived
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  Duane Wessels and the University of California San Diego.  Please
- *  see the COPYRIGHT file for full details.  Squid incorporates
- *  software developed and/or copyrighted by other sources.  Please see
- *  the CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -354,22 +354,23 @@ ftpListingStart(FtpStateData * ftpState)
     wordlist *w;
     char *dirup;
     int i, j, k;
+    char *title;
     storeBuffer(e);
     storeAppendPrintf(e, "<!-- HTML listing generated by Squid %s -->\n",
 	version_string);
     storeAppendPrintf(e, "<!-- %s -->\n", mkrfc1123(squid_curtime));
     storeAppendPrintf(e, "<HTML><HEAD><TITLE>\n");
     storeAppendPrintf(e, "FTP Directory: %s\n",
-	ftpState->title_url);
+	html_quote(ftpState->title_url));
     storeAppendPrintf(e, "</TITLE>\n");
     if (ftpState->flags.use_base)
 	storeAppendPrintf(e, "<BASE HREF=\"%s\">\n",
-	    ftpState->base_href);
+	    html_quote(ftpState->base_href));
     storeAppendPrintf(e, "</HEAD><BODY>\n");
     if (ftpState->cwd_message) {
 	storeAppendPrintf(e, "<PRE>\n");
 	for (w = ftpState->cwd_message; w; w = w->next)
-	    storeAppendPrintf(e, "%s\n", w->key);
+	    storeAppendPrintf(e, "%s\n", html_quote(w->key));
 	storeAppendPrintf(e, "</PRE>\n");
 	storeAppendPrintf(e, "<HR>\n");
 	wordlistDestroy(&ftpState->cwd_message);
@@ -378,25 +379,26 @@ ftpListingStart(FtpStateData * ftpState)
     storeAppendPrintf(e, "FTP Directory: ");
     /* "ftp://" == 6 characters */
     assert(strlen(ftpState->title_url) >= 6);
-    for (i = 6, j = 0; ftpState->title_url[i]; j = i) {
+    title = html_quote(ftpState->title_url);
+    for (i = 6, j = 0; title[i]; j = i) {
 	storeAppendPrintf(e, "<A HREF=\"");
-	i += strcspn(&ftpState->title_url[i], "/");
-	if (ftpState->title_url[i] == '/')
+	i += strcspn(&title[i], "/");
+	if (title[i] == '/')
 	    i++;
 	for (k = 0; k < i; k++)
-	    storeAppendPrintf(e, "%c", ftpState->title_url[k]);
+	    storeAppendPrintf(e, "%c", title[k]);
 	storeAppendPrintf(e, "\">");
 	for (k = j; k < i - 1; k++)
-	    storeAppendPrintf(e, "%c", ftpState->title_url[k]);
+	    storeAppendPrintf(e, "%c", title[k]);
 	if (ftpState->title_url[k] != '/')
-	    storeAppendPrintf(e, "%c", ftpState->title_url[k++]);
+	    storeAppendPrintf(e, "%c", title[k++]);
 	storeAppendPrintf(e, "</A>");
 	if (k < i)
-	    storeAppendPrintf(e, "%c", ftpState->title_url[k++]);
+	    storeAppendPrintf(e, "%c", title[k++]);
 	if (i == j) {
 	    /* Error guard, or "assert" */
 	    storeAppendPrintf(e, "ERROR: Failed to parse URL: %s\n",
-		ftpState->title_url);
+		html_quote(ftpState->title_url));
 	    debug(9, 0) ("Failed to parse URL: %s\n", ftpState->title_url);
 	    break;
 	}
@@ -422,11 +424,10 @@ ftpListingFinish(FtpStateData * ftpState)
     }
     storeAppendPrintf(e, "<HR>\n");
     storeAppendPrintf(e, "<ADDRESS>\n");
-    storeAppendPrintf(e, "Generated %s by %s (<a href=\"http://squid.nlanr.net/Squid/\">%s</a>)\n",
+    storeAppendPrintf(e, "Generated %s by %s (%s)\n",
 	mkrfc1123(squid_curtime),
 	getMyHostname(),
-	full_appname_string,
-	version_string);
+	full_appname_string);
     storeAppendPrintf(e, "</ADDRESS></BODY></HTML>\n");
     storeBufferFlush(e);
 }
@@ -568,6 +569,8 @@ ftpListParseParts(const char *buf, struct _ftp_flags flags)
 	ct = buf + 1;
 	p->type = 0;
 	while (ct && *ct) {
+	    long lt;
+	    time_t t;
 	    switch (*ct) {
 	    case '\t':
 		sscanf(ct + 1, "%[^,]", sbuf);
@@ -577,8 +580,10 @@ ftpListParseParts(const char *buf, struct _ftp_flags flags)
 		sscanf(ct + 1, "%d", &(p->size));
 		break;
 	    case 'm':
-		sscanf(ct + 1, "%d", &i);
-		p->date = xstrdup(ctime((time_t *) & i));
+		if (1 != sscanf(ct + 1, "%ld", &lt))
+		    break;
+		t = lt;
+		p->date = xstrdup(ctime(&t));
 		*(strstr(p->date, "\n")) = '\0';
 		break;
 	    case '/':
@@ -645,7 +650,7 @@ ftpHtmlifyListEntry(char *line, FtpStateData * ftpState)
 	return html;
     }
     /* Handle builtin <dirup> */
-    if (!strcmp(line, "<internal-dirup>")) {
+    if (strcmp(line, "<internal-dirup>") == 0) {
 	/* <A HREF="{href}">{icon}</A> <A HREF="{href}">{text}</A> {link} */
 	snprintf(icon, 2048, "<IMG BORDER=0 SRC=\"%s\" ALT=\"%-6s\">",
 	    mimeGetIconURL("internal-dirup"),
@@ -714,10 +719,13 @@ ftpHtmlifyListEntry(char *line, FtpStateData * ftpState)
 	    mimeGetIconURL("internal-link"),
 	    "[LINK]");
 	/* sometimes there is an 'l' flag, but no "->" link */
-	if (parts->link)
+	if (parts->link) {
+	    char *link2 = xstrdup(html_quote(rfc1738_escape(parts->link)));
 	    snprintf(link, 2048, " -> <A HREF=\"%s\">%s</A>",
-		rfc1738_escape(parts->link),
-		parts->link);
+		link2,
+		html_quote(parts->link));
+	    safe_free(link2);
+	}
 	break;
     case '\0':
 	snprintf(icon, 2048, "<IMG BORDER=0 SRC=\"%s\" ALT=\"%-6s\">",
@@ -725,7 +733,7 @@ ftpHtmlifyListEntry(char *line, FtpStateData * ftpState)
 	    "[UNKNOWN]");
 	snprintf(chdir, 2048, " <A HREF=\"%s/;type=d\"><IMG BORDER=0 SRC=\"%s\" "
 	    "ALT=\"[DIR]\"></A>",
-	    rfc1738_escape(parts->name),
+	    rfc1738_escape_part(parts->name),
 	    mimeGetIconURL("internal-dir"));
 	break;
     case '-':
@@ -752,13 +760,13 @@ ftpHtmlifyListEntry(char *line, FtpStateData * ftpState)
     if (parts->type != '\0') {
 	snprintf(html, 8192, "<A HREF=\"%s\">%s</A> <A HREF=\"%s\">%s</A>%s "
 	    "%s%8s%s%s%s%s\n",
-	    href, icon, href, text, dots_fill(strlen(text)),
+	    href, icon, href, html_quote(text), dots_fill(strlen(text)),
 	    parts->date, size, chdir, view, download, link);
     } else {
 	/* Plain listing. {icon} {text} ... {chdir}{view}{download} */
 	snprintf(html, 8192, "<A HREF=\"%s\">%s</A> <A HREF=\"%s\">%s</A>%s "
 	    "%s%s%s%s\n",
-	    href, icon, href, text, dots_fill(strlen(text)),
+	    href, icon, href, html_quote(text), dots_fill(strlen(text)),
 	    chdir, view, download, link);
     }
     ftpListPartsFree(&parts);
@@ -870,15 +878,15 @@ ftpDataRead(int fd, void *data)
     read_sz = delayBytesWanted(delay_id, 1, read_sz);
 #endif
     memset(ftpState->data.buf + ftpState->data.offset, '\0', read_sz);
-    Counter.syscalls.sock.reads++;
+    statCounter.syscalls.sock.reads++;
     len = read(fd, ftpState->data.buf + ftpState->data.offset, read_sz);
     if (len > 0) {
 	fd_bytes(fd, len, FD_READ);
 #if DELAY_POOLS
 	delayBytesIn(delay_id, len);
 #endif
-	kb_incr(&Counter.server.all.kbytes_in, len);
-	kb_incr(&Counter.server.ftp.kbytes_in, len);
+	kb_incr(&statCounter.server.all.kbytes_in, len);
+	kb_incr(&statCounter.server.ftp.kbytes_in, len);
 	ftpState->data.offset += len;
     }
     debug(9, 5) ("ftpDataRead: FD %d, Read %d bytes\n", fd, len);
@@ -892,7 +900,7 @@ ftpDataRead(int fd, void *data)
 	ftpListingStart(ftpState);
     }
     if (len < 0) {
-	debug(50, 1) ("ftpDataRead: read error: %s\n", xstrerror());
+	debug(50, ignoreErrno(errno) ? 3 : 1) ("ftpDataRead: read error: %s\n", xstrerror());
 	if (ignoreErrno(errno)) {
 	    commSetSelect(fd,
 		COMM_SELECT_READ,
@@ -1039,8 +1047,8 @@ ftpStart(FwdState * fwd)
     const cache_key *key = NULL;
     cbdataAdd(ftpState, cbdataXfree, 0);
     debug(9, 3) ("ftpStart: '%s'\n", url);
-    Counter.server.all.requests++;
-    Counter.server.ftp.requests++;
+    statCounter.server.all.requests++;
+    statCounter.server.ftp.requests++;
     storeLockObject(entry);
     ftpState->entry = entry;
     ftpState->request = requestLink(request);
@@ -1048,7 +1056,12 @@ ftpStart(FwdState * fwd)
     ftpState->data.fd = -1;
     ftpState->size = -1;
     ftpState->mdtm = -1;
-    ftpState->flags.pasv_supported = !fwd->flags.ftp_pasv_failed;
+    if (!Config.Ftp.passive)
+	ftpState->flags.rest_supported = 0;
+    else if (fwd->flags.ftp_pasv_failed)
+	ftpState->flags.pasv_supported = 0;
+    else
+	ftpState->flags.pasv_supported = 1;
     ftpState->flags.rest_supported = 1;
     ftpState->fwd = fwd;
     comm_add_close_handler(fd, ftpStateFree, ftpState);
@@ -1118,8 +1131,8 @@ ftpWriteCommandCallback(int fd, char *bufnotused, size_t size, int errflag, void
     debug(9, 7) ("ftpWriteCommandCallback: wrote %d bytes\n", size);
     if (size > 0) {
 	fd_bytes(fd, size, FD_WRITE);
-	kb_incr(&Counter.server.all.kbytes_out, size);
-	kb_incr(&Counter.server.ftp.kbytes_out, size);
+	kb_incr(&statCounter.server.all.kbytes_out, size);
+	kb_incr(&statCounter.server.ftp.kbytes_out, size);
     }
     if (errflag == COMM_ERR_CLOSING)
 	return;
@@ -1232,18 +1245,18 @@ ftpReadControlReply(int fd, void *data)
 	return;
     }
     assert(ftpState->ctrl.offset < ftpState->ctrl.size);
-    Counter.syscalls.sock.reads++;
+    statCounter.syscalls.sock.reads++;
     len = read(fd,
 	ftpState->ctrl.buf + ftpState->ctrl.offset,
 	ftpState->ctrl.size - ftpState->ctrl.offset);
     if (len > 0) {
 	fd_bytes(fd, len, FD_READ);
-	kb_incr(&Counter.server.all.kbytes_in, len);
-	kb_incr(&Counter.server.ftp.kbytes_in, len);
+	kb_incr(&statCounter.server.all.kbytes_in, len);
+	kb_incr(&statCounter.server.ftp.kbytes_in, len);
     }
     debug(9, 5) ("ftpReadControlReply: FD %d, Read %d bytes\n", fd, len);
     if (len < 0) {
-	debug(50, 1) ("ftpReadControlReply: read error: %s\n", xstrerror());
+	debug(50, ignoreErrno(errno) ? 3 : 1) ("ftpReadControlReply: read error: %s\n", xstrerror());
 	if (ignoreErrno(errno)) {
 	    ftpScheduleReadControlReply(ftpState, 0);
 	} else {
@@ -1299,11 +1312,15 @@ ftpHandleControlReply(FtpStateData * ftpState)
 	xmemmove(ftpState->ctrl.buf, ftpState->ctrl.buf + bytes_used,
 	    ftpState->ctrl.offset);
     }
-    /* Find the last line of the reply message */
+    /* Move the last line of the reply message to ctrl.last_reply */
     for (W = &ftpState->ctrl.message; (*W)->next; W = &(*W)->next);
     safe_free(ftpState->ctrl.last_reply);
-    ftpState->ctrl.last_reply = (*W)->key;
-    safe_free(*W);
+    ftpState->ctrl.last_reply = xstrdup((*W)->key);
+    wordlistDestroy(W);
+    /* Copy the rest of the message to cwd_message to be printed in
+     * error messages
+     */
+    wordlistAddWl(&ftpState->cwd_message, ftpState->ctrl.message);
     debug(9, 8) ("ftpHandleControlReply: state=%d, code=%d\n", ftpState->state,
 	ftpState->ctrl.replycode);
     FTP_SM_FUNCS[ftpState->state] (ftpState);
@@ -1324,10 +1341,6 @@ ftpReadWelcome(FtpStateData * ftpState)
 	if (ftpState->ctrl.message) {
 	    if (strstr(ftpState->ctrl.message->key, "NetWare"))
 		ftpState->flags.skip_whitespace = 1;
-	    if (ftpState->cwd_message)
-		wordlistDestroy(&ftpState->cwd_message);
-	    ftpState->cwd_message = ftpState->ctrl.message;
-	    ftpState->ctrl.message = NULL;
 	}
 	ftpSendUser(ftpState);
     } else if (code == 120) {
@@ -1381,12 +1394,6 @@ ftpReadPass(FtpStateData * ftpState)
     int code = ftpState->ctrl.replycode;
     debug(9, 3) ("ftpReadPass\n");
     if (code == 230) {
-	if (ftpState->ctrl.message) {
-	    if (ftpState->cwd_message)
-		wordlistDestroy(&ftpState->cwd_message);
-	    ftpState->cwd_message = ftpState->ctrl.message;
-	    ftpState->ctrl.message = NULL;
-	}
 	ftpSendType(ftpState);
     } else {
 	ftpFail(ftpState);
@@ -1475,7 +1482,7 @@ ftpTraverseDirectory(FtpStateData * ftpState)
     w = ftpState->pathcomps;
     ftpState->filepath = w->key;
     ftpState->pathcomps = w->next;
-    xfree(w);
+    memFree(w, MEM_WORDLIST);
     /* Check if we are to CWD or RETR */
     if (ftpState->pathcomps != NULL || ftpState->flags.isdir) {
 	ftpSendCwd(ftpState);
@@ -1512,6 +1519,7 @@ ftpReadCwd(FtpStateData * ftpState)
     if (code >= 200 && code < 300) {
 	/* CWD OK */
 	ftpUnhack(ftpState);
+	/* Reset cwd_message to only include the last message */
 	if (ftpState->cwd_message)
 	    wordlistDestroy(&ftpState->cwd_message);
 	ftpState->cwd_message = ftpState->ctrl.message;
@@ -1641,6 +1649,20 @@ ftpSendPasv(FtpStateData * ftpState)
     int fd;
     struct sockaddr_in addr;
     socklen_t addr_len;
+    if (ftpState->request->method == METHOD_HEAD) {
+	/* Terminate here for HEAD requests */
+	ftpAppendSuccessHeader(ftpState);
+	storeTimestampsSet(ftpState->entry);
+	/*
+	 * On rare occasions I'm seeing the entry get aborted after
+	 * ftpReadControlReply() and before here, probably when
+	 * trying to write to the client.
+	 */
+	if (!EBIT_TEST(ftpState->entry->flags, ENTRY_ABORTED))
+	    fwdComplete(ftpState->fwd);
+	ftpSendQuit(ftpState);
+	return;
+    }
     if (ftpState->data.fd >= 0) {
 	if (!ftpState->flags.datachannel_hack) {
 	    /* We are already connected, reuse this connection. */
@@ -1812,7 +1834,7 @@ ftpOpenListenSocket(FtpStateData * ftpState, int fallback)
 	return -1;
     }
     ftpState->data.fd = fd;
-    ftpState->data.port = comm_local_port(fd);;
+    ftpState->data.port = comm_local_port(fd);
     ftpState->data.host = NULL;
     return fd;
 }
@@ -1862,10 +1884,14 @@ static void
 ftpAcceptDataConnection(int fd, void *data)
 {
     FtpStateData *ftpState = data;
-    struct sockaddr_in peer, me;
+    struct sockaddr_in my_peer, me;
     debug(9, 3) ("ftpAcceptDataConnection\n");
 
-    fd = comm_accept(fd, &peer, &me);
+    if (EBIT_TEST(ftpState->entry->flags, ENTRY_ABORTED)) {
+	comm_close(ftpState->ctrl.fd);
+	return;
+    }
+    fd = comm_accept(fd, &my_peer, &me);
     if (fd < 0) {
 	debug(9, 1) ("ftpHandleDataAccept: comm_accept(%d): %s", fd, xstrerror());
 	/* XXX Need to set error message */
@@ -1876,8 +1902,8 @@ ftpAcceptDataConnection(int fd, void *data)
     comm_close(ftpState->data.fd);
     debug(9, 3) ("ftpAcceptDataConnection: Connected data socket on FD %d\n", fd);
     ftpState->data.fd = fd;
-    ftpState->data.port = ntohs(peer.sin_port);
-    ftpState->data.host = xstrdup(inet_ntoa(peer.sin_addr));
+    ftpState->data.port = ntohs(my_peer.sin_port);
+    ftpState->data.host = xstrdup(inet_ntoa(my_peer.sin_addr));
     commSetTimeout(ftpState->ctrl.fd, -1, NULL, NULL);
     commSetTimeout(ftpState->data.fd, Config.Timeout.read, ftpTimeout,
 	ftpState);
@@ -2442,6 +2468,7 @@ ftpAppendSuccessHeader(FtpStateData * ftpState)
     StoreEntry *e = ftpState->entry;
     StoreEntry *pe = NULL;
     http_reply *reply = e->mem_obj->reply;
+    http_version_t version;
     if (ftpState->flags.http_header_sent)
 	return;
     ftpState->flags.http_header_sent = 1;
@@ -2473,12 +2500,14 @@ ftpAppendSuccessHeader(FtpStateData * ftpState)
 	HttpHdrRangeSpec range_spec;
 	range_spec.offset = ftpState->restarted_offset;
 	range_spec.length = ftpState->size - ftpState->restarted_offset;
-	httpReplySetHeaders(reply, 1.0, HTTP_PARTIAL_CONTENT, "Gatewaying",
+	httpBuildVersion(&version, 1, 0);
+	httpReplySetHeaders(reply, version, HTTP_PARTIAL_CONTENT, "Gatewaying",
 	    mime_type, ftpState->size - ftpState->restarted_offset, ftpState->mdtm, -2);
 	httpHeaderAddContRange(&reply->header, range_spec, ftpState->size);
     } else {
 	/* Full reply */
-	httpReplySetHeaders(reply, 1.0, HTTP_OK, "Gatewaying",
+	httpBuildVersion(&version, 1, 0);
+	httpReplySetHeaders(reply, version, HTTP_OK, "Gatewaying",
 	    mime_type, ftpState->size, ftpState->mdtm, -2);
     }
     /* additional info */
