@@ -1,21 +1,21 @@
 
 /*
- * $Id: main.c,v 1.302.2.13 2000/01/05 23:45:31 wessels Exp $
+ * $Id: main.c,v 1.322.2.3 2001/01/12 00:51:50 wessels Exp $
  *
  * DEBUG: section 1     Startup and Main Loop
  * AUTHOR: Harvest Derived
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  Duane Wessels and the University of California San Diego.  Please
- *  see the COPYRIGHT file for full details.  Squid incorporates
- *  software developed and/or copyrighted by other sources.  Please see
- *  the CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -71,6 +71,9 @@ extern void log_trace_init(char *);
 #endif
 static EVH SquidShutdown;
 static void mainSetCwd(void);
+static int checkRunningPid(void);
+
+static const char *squid_start_script = "squid_start";
 
 #if TEST_ACCESS
 #include "test_access.c"
@@ -98,6 +101,7 @@ usage(void)
 	"       -F        Don't serve any requests until store is rebuilt.\n"
 	"       -N        No daemon mode.\n"
 	"       -R        Do not set REUSEADDR on port.\n"
+	"       -S        Double-check swap during rebuild.\n"
 	"       -V        Virtual host httpd-accelerator.\n"
 	"       -X        Force full debugging.\n"
 	"       -Y        Only return UDP_HIT or UDP_MISS_NOFETCH during fast reload.\n",
@@ -333,8 +337,9 @@ mainReconfigure(void)
 #if USE_WCCP
     wccpConnectionClose();
 #endif
+#if USE_DNSSERVERS
     dnsShutdown();
-#if !USE_DNSSERVERS
+#else
     idnsShutdown();
 #endif
     redirectShutdown();
@@ -347,8 +352,9 @@ mainReconfigure(void)
     ipcache_restart();		/* clear stuck entries */
     fqdncache_restart();	/* sigh, fqdncache too */
     errorInitialize();		/* reload error pages */
+#if USE_DNSSERVERS
     dnsInit();
-#if !USE_DNSSERVERS
+#else
     idnsInit();
 #endif
     redirectInit();
@@ -374,12 +380,26 @@ static void
 mainRotate(void)
 {
     icmpClose();
+#if USE_DNSSERVERS
+    dnsShutdown();
+#endif
+    redirectShutdown();
+    authenticateShutdown();
     _db_rotate_log();		/* cache.log */
     storeDirWriteCleanLogs(1);
     storeLogRotate();		/* store.log */
     accessLogRotate();		/* access.log */
     useragentRotateLog();	/* useragent.log */
+    refererRotateLog();		/* referer.log */
+#if WIP_FWD_LOG
+    fwdLogRotate();
+#endif
     icmpOpen();
+#if USE_DNSSERVERS
+    dnsInit();
+#endif
+    redirectInit();
+    authenticateInit();
 }
 
 static void
@@ -462,13 +482,15 @@ mainInitialize(void)
 	disk_init();		/* disk_init must go before ipcache_init() */
     ipcache_init();
     fqdncache_init();
+#if USE_DNSSERVERS
     dnsInit();
-#if !USE_DNSSERVERS
+#else
     idnsInit();
 #endif
     redirectInit();
     authenticateInit();
     useragentOpenLog();
+    refererOpenLog();
     httpHeaderInitModule();	/* must go before any header processing (e.g. the one in errorInitialize) */
     httpReplyInitModule();	/* must go before accepting replies */
     errorInitialize();
@@ -484,10 +506,9 @@ mainInitialize(void)
 #endif
 
     if (!configured_once) {
-#if USE_ASYNC_IO
-	aioInit();
-#endif
+#if USE_UNLINKD
 	unlinkdInit();
+#endif
 	urlInitialize();
 	cachemgrInit();
 	statInit();
@@ -603,17 +624,21 @@ main(int argc, char **argv)
 	if (!ConfigFile)
 	    ConfigFile = xstrdup(DefaultConfigFile);
 	assert(!configured_once);
+	memInit();		/* memInit is required for config parsing */
 	cbdataInit();
 #if USE_LEAKFINDER
 	leakInit();
 #endif
-	memInit();		/* memInit is required for config parsing */
 	eventInit();		/* eventInit() is required for config parsing */
+	storeFsInit();		/* required for config parsing */
 	parse_err = parseConfigFile(ConfigFile);
 
 	if (opt_parse_cfg_only)
 	    return parse_err;
     }
+    if (-1 == opt_send_signal)
+	if (checkRunningPid())
+	    exit(1);
 
 #if TEST_ACCESS
     comm_init();
@@ -673,14 +698,15 @@ main(int argc, char **argv)
 	} else if (do_shutdown) {
 	    time_t wait = do_shutdown > 0 ? (int) Config.shutdownLifetime : 0;
 	    debug(1, 1) ("Preparing for shutdown after %d requests\n",
-		Counter.client_http.requests);
+		statCounter.client_http.requests);
 	    debug(1, 1) ("Waiting %d seconds for active connections to finish\n",
 		wait);
 	    do_shutdown = 0;
 	    shutting_down = 1;
 	    serverConnectionsClose();
+#if USE_DNSSERVERS
 	    dnsShutdown();
-#if !USE_DNSSERVERS
+#else
 	    idnsShutdown();
 #endif
 	    redirectShutdown();
@@ -741,6 +767,57 @@ sendSignal(void)
     exit(0);
 }
 
+/*
+ * This function is run when Squid is in daemon mode, just
+ * before the parent forks and starts up the child process.
+ * It can be used for admin-specific tasks, such as notifying
+ * someone that Squid is (re)started.
+ */
+static void
+mainStartScript(const char *prog)
+{
+    char script[SQUID_MAXPATHLEN];
+    char *t;
+    size_t sl = 0;
+    pid_t cpid;
+    pid_t rpid;
+    xstrncpy(script, prog, MAXPATHLEN);
+    if ((t = strrchr(script, '/'))) {
+	*(++t) = '\0';
+	sl = strlen(script);
+    }
+    xstrncpy(&script[sl], squid_start_script, MAXPATHLEN - sl);
+    if ((cpid = fork()) == 0) {
+	/* child */
+	execl(script, squid_start_script, 0);
+	_exit(0);
+    } else {
+	do {
+#ifdef _SQUID_NEXT_
+	    union wait status;
+	    rpid = wait3(&status, 0, NULL);
+#else
+	    int status;
+	    rpid = waitpid(-1, &status, 0);
+#endif
+	} while (rpid != cpid);
+    }
+}
+
+static int
+checkRunningPid(void)
+{
+    pid_t pid;
+    debug_log = stderr;
+    pid = readPidFile();
+    if (pid < 2)
+	return 0;
+    if (kill(pid, 0) < 0)
+	return 0;
+    debug(0, 0) ("Squid is already running!  Process ID %d\n", pid);
+    return 1;
+}
+
 static void
 watch_child(char *argv[])
 {
@@ -755,6 +832,7 @@ watch_child(char *argv[])
 #endif
     pid_t pid;
     int i;
+    int nullfd;
     if (*(argv[0]) == '(')
 	return;
     openlog(appname, LOG_PID | LOG_NDELAY | LOG_CONS, LOG_LOCAL4);
@@ -764,23 +842,42 @@ watch_child(char *argv[])
 	exit(0);
     if (setsid() < 0)
 	syslog(LOG_ALERT, "setsid failed: %s", xstrerror());
+    closelog();
 #ifdef TIOCNOTTY
-    if ((i = open("/dev/tty", O_RDWR)) >= 0) {
+    if ((i = open("/dev/tty", O_RDWR | O_TEXT)) >= 0) {
 	ioctl(i, TIOCNOTTY, NULL);
 	close(i);
     }
 #endif
-    for (i = 0; i < Squid_MaxFD; i++)
+
+
+    /*
+     * RBCOLLINS - if cygwin stackdumps when squid is run without
+     * -N, check the cygwin1.dll version, it needs to be AT LEAST
+     * 1.1.3.  execvp had a bit overflow error in a loop..
+     */
+    /* Connect stdio to /dev/null in daemon mode */
+    nullfd = open("/dev/null", O_RDWR | O_TEXT);
+    dup2(nullfd, 0);
+    if (opt_debug_stderr < 0) {
+	dup2(nullfd, 1);
+	dup2(nullfd, 2);
+    }
+    /* Close all else */
+    for (i = 3; i < Squid_MaxFD; i++)
 	close(i);
     for (;;) {
+	mainStartScript(argv[0]);
 	if ((pid = fork()) == 0) {
 	    /* child */
+	    openlog(appname, LOG_PID | LOG_NDELAY | LOG_CONS, LOG_LOCAL4);
 	    prog = xstrdup(argv[0]);
 	    argv[0] = xstrdup("(squid)");
 	    execvp(prog, argv);
 	    syslog(LOG_ALERT, "execvp failed: %s", xstrerror());
 	}
 	/* parent */
+	openlog(appname, LOG_PID | LOG_NDELAY | LOG_CONS, LOG_LOCAL4);
 	syslog(LOG_NOTICE, "Squid Parent: child process %d started", pid);
 	time(&start);
 	squid_signal(SIGINT, SIG_IGN, SA_RESTART);
@@ -839,22 +936,22 @@ SquidShutdown(void *unused)
 #endif
     releaseServerSockets();
     commCloseAllSockets();
+#if USE_UNLINKD
     unlinkdClose();
-#if USE_ASYNC_IO
-    aioSync();			/* flush pending object writes / unlinks */
 #endif
+    storeDirSync();		/* Flush pending object writes/unlinks */
     storeDirWriteCleanLogs(0);
     PrintRusage();
     dumpMallocStats();
-#if USE_ASYNC_IO
-    aioSync();			/* flush log writes */
-#endif
+    storeDirSync();		/* Flush log writes */
     storeLogClose();
     accessLogClose();
-#if USE_ASYNC_IO
-    aioSync();			/* flush log close */
+#if WIP_FWD_LOG
+    fwdUninit();
 #endif
+    storeDirSync();		/* Flush log close */
 #if PURIFY || XMALLOC_TRACE
+    storeFsDone();
     configFreeMemory();
     storeFreeMemory();
     /*stmemFreeMemory(); */
