@@ -1,21 +1,21 @@
 
 /*
- * $Id: neighbors.c,v 1.275.2.4 1999/10/21 19:11:29 wessels Exp $
+ * $Id: neighbors.c,v 1.288.2.2 2001/01/12 00:51:50 wessels Exp $
  *
  * DEBUG: section 15    Neighbor Routines
  * AUTHOR: Harvest Derived
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  Duane Wessels and the University of California San Diego.  Please
- *  see the COPYRIGHT file for full details.  Squid incorporates
- *  software developed and/or copyrighted by other sources.  Please see
- *  the CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,9 +48,9 @@ static void neighborAliveHtcp(peer *, const MemObject *, const htcpReplyData *);
 static void neighborCountIgnored(peer *);
 static void peerRefreshDNS(void *);
 static IPH peerDNSConfigure;
-static EVH peerCheckConnect;
-static IPH peerCheckConnect2;
-static CNCB peerCheckConnectDone;
+static void peerProbeConnect(peer *);
+static IPH peerProbeConnect2;
+static CNCB peerProbeConnectDone;
 static void peerCountMcastPeersDone(void *data);
 static void peerCountMcastPeersStart(void *data);
 static void peerCountMcastPeersSchedule(peer * p, time_t when);
@@ -146,6 +146,7 @@ peerAllowedToUse(const peer * p, request_t * request)
 	return do_ping;
     if (p->access == NULL)
 	return do_ping;
+    memset(&checklist, '\0', sizeof(checklist));
     checklist.src_addr = request->client_addr;
     checklist.my_addr = request->my_addr;
     checklist.my_port = request->my_port;
@@ -163,15 +164,19 @@ peerWouldBePinged(const peer * p, request_t * request)
 	return 0;
     if (p->options.mcast_responder)
 	return 0;
+    if (p->n_addresses == 0)
+	return 0;
+    if (p->icp.port == 0)
+	return 0;
     /* the case below seems strange, but can happen if the
      * URL host is on the other side of a firewall */
     if (p->type == PEER_SIBLING)
 	if (!request->flags.hierarchical)
 	    return 0;
-    if (p->icp.port == echo_port)
-	if (!neighborUp(p))
-	    return 0;
-    if (p->n_addresses == 0)
+    /* Ping dead peers every timeout interval */
+    if (squid_curtime - p->stats.last_query > Config.Timeout.deadPeer)
+	return 1;
+    if (!neighborUp(p))
 	return 0;
     return 1;
 }
@@ -258,6 +263,18 @@ getRoundRobinParent(request_t * request)
     return q;
 }
 
+/* This gets called every 5 minutes to clear the round-robin counter. */
+void
+peerClearRR(void *data)
+{
+    peer *p = data;
+    p->rr_count -= p->rr_lastcount;
+    if (p->rr_count < 0)
+	p->rr_count = 0;
+    p->rr_lastcount = p->rr_count;
+    eventAdd("peerClearRR", peerClearRR, p, 5 * 60, 0);
+}
+
 peer *
 getDefaultParent(request_t * request)
 {
@@ -331,9 +348,28 @@ neighbors_open(int fd)
     struct sockaddr_in name;
     socklen_t len = sizeof(struct sockaddr_in);
     struct servent *sep = NULL;
+    const char *me = getMyHostname();
+    peer *this;
+    peer *next;
     memset(&name, '\0', sizeof(struct sockaddr_in));
     if (getsockname(fd, (struct sockaddr *) &name, &len) < 0)
 	debug(15, 1) ("getsockname(%d,%p,%p) failed.\n", fd, &name, &len);
+    for (this = Config.peers; this; this = next) {
+	sockaddr_in_list *s;
+	next = this->next;
+	if (0 != strcmp(this->host, me))
+	    continue;
+	for (s = Config.Sockaddr.http; s; s = s->next) {
+	    if (this->http_port != ntohs(s->s.sin_port))
+		continue;
+	    debug(15, 1) ("WARNING: Peer looks like this host\n");
+	    debug(15, 1) ("         Ignoring %s %s/%d/%d\n",
+		neighborTypeStr(this), this->host, this->http_port,
+		this->icp.port);
+	    neighborRemove(this);
+	}
+    }
+
     peerRefreshDNS((void *) 1);
     if (0 == echo_hdr.opcode) {
 	echo_hdr.opcode = ICP_SECHO;
@@ -372,6 +408,8 @@ neighborsUdpPing(request_t * request,
     icp_common_t *query;
     int queries_sent = 0;
     int peers_pinged = 0;
+    int parent_timeout = 0, parent_exprep = 0;
+    int sibling_timeout = 0, sibling_exprep = 0;
 
     if (Config.peers == NULL)
 	return 0;
@@ -381,8 +419,7 @@ neighborsUdpPing(request_t * request,
     mem->start_ping = current_time;
     mem->ping_reply_callback = callback;
     mem->ircb_data = callback_data;
-    *timeout = 0.0;
-    reqnum = icpSetCacheKey(entry->key);
+    reqnum = icpSetCacheKey(entry->hash.key);
     for (i = 0, p = first_ping; i++ < Config.npeers; p = p->next) {
 	if (p == NULL)
 	    p = Config.peers;
@@ -394,7 +431,7 @@ neighborsUdpPing(request_t * request,
 	    p->host, url);
 	if (p->type == PEER_MULTICAST)
 	    mcastSetTtl(theOutIcpConnection, p->mcast.ttl);
-	debug(15, 3) ("neighborsUdpPing: key = '%s'\n", storeKeyText(entry->key));
+	debug(15, 3) ("neighborsUdpPing: key = '%s'\n", storeKeyText(entry->hash.key));
 	debug(15, 3) ("neighborsUdpPing: reqnum = %d\n", reqnum);
 
 #if USE_HTCP
@@ -434,28 +471,15 @@ neighborsUdpPing(request_t * request,
 	     */
 	    p->stats.last_reply = squid_curtime;
 	    (*exprep) += p->mcast.n_replies_expected;
-	} else if (squid_curtime - p->stats.last_query > Config.Timeout.deadPeer) {
-	    /*
-	     * fake a recent reply if its been a long time since our
-	     * last query
-	     */
-	    p->stats.last_reply = squid_curtime;
-	    /*
-	     * We used to not expect a reply in this case; we assumed
-	     * the peer was DEAD if we hadn't queried it in a long
-	     * time.  However, the number of people whining to
-	     * squid-users that ICP is broken became unbearable.  They
-	     * tried a single request which, to their amazement, was
-	     * forwarded directly to the origin server, even thought
-	     * they KNEW it was in a neighbor cache.  Ok, I give up, you
-	     * win!
-	     */
-	    (*exprep)++;
-	    (*timeout) += 1000;
 	} else if (neighborUp(p)) {
 	    /* its alive, expect a reply from it */
-	    (*exprep)++;
-	    (*timeout) += p->stats.rtt;
+	    if (neighborType(p, request) == PEER_PARENT) {
+		parent_exprep++;
+		parent_timeout += p->stats.rtt;
+	    } else {
+		sibling_exprep++;
+		sibling_timeout += p->stats.rtt;
+	    }
 	} else {
 	    /* Neighbor is dead; ping it anyway, but don't expect a reply */
 	    /* log it once at the threshold */
@@ -467,6 +491,8 @@ neighborsUdpPing(request_t * request,
 	    }
 	}
 	p->stats.last_query = squid_curtime;
+	if (p->stats.probe_start == 0)
+	    p->stats.probe_start = squid_curtime;
     }
     if ((first_ping = first_ping->next) == NULL)
 	first_ping = Config.peers;
@@ -502,15 +528,23 @@ neighborsUdpPing(request_t * request,
 	}
     }
 #endif
+    /*
+     * How many replies to expect?
+     */
+    *exprep = parent_exprep + sibling_exprep;
+
     /*
      * If there is a configured timeout, use it
      */
     if (Config.Timeout.icp_query)
 	*timeout = Config.Timeout.icp_query;
     else {
-	if (*exprep > 0)
-	    (*timeout) = 2 * (*timeout) / (*exprep);
-	else
+	if (*exprep > 0) {
+	    if (parent_exprep)
+		*timeout = 2 * parent_timeout / parent_exprep;
+	    else
+		*timeout = 2 * sibling_timeout / sibling_exprep;
+	} else
 	    *timeout = 2000;	/* 2 seconds */
 	if (Config.Timeout.icp_query_max)
 	    if (*timeout > Config.Timeout.icp_query_max)
@@ -633,6 +667,7 @@ neighborAlive(peer * p, const MemObject * mem, const icp_common_t * header)
 	p->stats.logged_state = PEER_ALIVE;
     }
     p->stats.last_reply = squid_curtime;
+    p->stats.probe_start = 0;
     p->stats.pings_acked++;
     if ((icp_opcode) header->opcode <= ICP_END)
 	p->icp.counts[header->opcode]++;
@@ -665,6 +700,7 @@ neighborAliveHtcp(peer * p, const MemObject * mem, const htcpReplyData * htcp)
 	p->stats.logged_state = PEER_ALIVE;
     }
     p->stats.last_reply = squid_curtime;
+    p->stats.probe_start = 0;
     p->stats.pings_acked++;
     p->htcp.counts[htcp->hit ? 1 : 0]++;
     p->htcp.version = htcp->version;
@@ -686,7 +722,6 @@ static void
 neighborIgnoreNonPeer(const struct sockaddr_in *from, icp_opcode opcode)
 {
     peer *np;
-    double x;
     for (np = non_peers; np; np = np->next) {
 	if (np->in_addr.sin_addr.s_addr != from->sin_addr.s_addr)
 	    continue;
@@ -704,13 +739,10 @@ neighborIgnoreNonPeer(const struct sockaddr_in *from, icp_opcode opcode)
 	np->next = non_peers;
 	non_peers = np;
     }
-    np->stats.ignored_replies++;
     np->icp.counts[opcode]++;
-    x = log(np->stats.ignored_replies) / log(10.0);
-    if (0.0 != x - (double) (int) x)
-	return;
-    debug(15, 1) ("WARNING: Ignored %d replies from non-peer %s\n",
-	np->stats.ignored_replies, np->host);
+    if (isPowTen(++np->stats.ignored_replies))
+	debug(15, 1) ("WARNING: Ignored %d replies from non-peer %s\n",
+	    np->stats.ignored_replies, np->host);
 }
 
 /* ignoreMulticastReply
@@ -878,11 +910,20 @@ peerFindByNameAndPort(const char *name, unsigned short port)
 int
 neighborUp(const peer * p)
 {
-    if (!p->tcp_up)
+    if (!p->tcp_up) {
+	peerProbeConnect((peer *) p);
 	return 0;
-    if (squid_curtime - p->stats.last_query > Config.Timeout.deadPeer)
+    }
+    if (p->options.no_query)
 	return 1;
-    if (p->stats.last_query - p->stats.last_reply > Config.Timeout.deadPeer)
+    if (p->stats.probe_start != 0 &&
+	squid_curtime - p->stats.probe_start > Config.Timeout.deadPeer)
+	return 0;
+    /*
+     * The peer can not be UP if we don't have any IP addresses
+     * for it. 
+     */
+    if (0 == p->n_addresses)
 	return 0;
     return 1;
 }
@@ -978,62 +1019,82 @@ peerRefreshDNS(void *data)
     eventAddIsh("peerRefreshDNS", peerRefreshDNS, NULL, 3600.0, 1);
 }
 
+void
+peerConnectFailed(peer * p)
+{
+    p->stats.last_connect_failure = squid_curtime;
+    if (!p->tcp_up) {
+	debug(15, 2) ("TCP connection to %s/%d dead\n", p->host, p->http_port);
+	return;
+    }
+    debug(15, 1) ("TCP connection to %s/%d failed\n", p->host, p->http_port);
+    p->tcp_up--;
+    if (!p->tcp_up) {
+	debug(15, 1) ("Detected DEAD %s: %s/%d/%d\n",
+	    neighborTypeStr(p),
+	    p->host, p->http_port, p->icp.port);
+	p->stats.logged_state = PEER_DEAD;
+    }
+}
+
+void
+peerConnectSucceded(peer * p)
+{
+    if (!p->tcp_up) {
+	debug(15, 2) ("TCP connection to %s/%d succeded\n", p->host, p->http_port);
+	debug(15, 1) ("Detected REVIVED %s: %s/%d/%d\n",
+	    neighborTypeStr(p),
+	    p->host, p->http_port, p->icp.port);
+	p->stats.logged_state = PEER_ALIVE;
+    }
+    p->tcp_up = PEER_TCP_MAGIC_COUNT;
+}
+
 /*
- * peerCheckConnect will NOT be called by eventRun if the peer/data
- * pointer becomes invalid.
+ * peerProbeConnect will be called on dead peers by neighborUp 
  */
 static void
-peerCheckConnect(void *data)
+peerProbeConnect(peer * p)
 {
-    peer *p = data;
     int fd;
+    if (p->test_fd != -1)
+	return;			/* probe already running */
+    if (squid_curtime - p->stats.last_connect_probe < Config.Timeout.connect)
+	return;			/* don't probe to often */
     fd = comm_open(SOCK_STREAM, 0, Config.Addrs.tcp_outgoing,
 	0, COMM_NONBLOCKING, p->host);
     if (fd < 0)
 	return;
     p->test_fd = fd;
-    ipcache_nbgethostbyname(p->host, peerCheckConnect2, p);
+    p->stats.last_connect_probe = squid_curtime;
+    ipcache_nbgethostbyname(p->host, peerProbeConnect2, p);
 }
 
 static void
-peerCheckConnect2(const ipcache_addrs * ianotused, void *data)
+peerProbeConnect2(const ipcache_addrs * ianotused, void *data)
 {
     peer *p = data;
     commConnectStart(p->test_fd,
 	p->host,
 	p->http_port,
-	peerCheckConnectDone,
+	peerProbeConnectDone,
 	p);
 }
 
 static void
-peerCheckConnectDone(int fd, int status, void *data)
+peerProbeConnectDone(int fd, int status, void *data)
 {
     peer *p = data;
     if (status == COMM_OK) {
-	p->tcp_up = PEER_TCP_MAGIC_COUNT;
-	debug(15, 1) ("TCP connection to %s/%d succeeded\n",
-	    p->host, p->http_port);
+	peerConnectSucceded(p);
     } else {
-	eventAdd("peerCheckConnect", peerCheckConnect, p, 60.0, 1);
+	peerConnectFailed(p);
     }
     comm_close(fd);
+    p->test_fd = -1;
     return;
 }
 
-void
-peerCheckConnectStart(peer * p)
-{
-    if (!p->tcp_up)
-	return;
-    debug(15, 1) ("TCP connection to %s/%d failed\n", p->host, p->http_port);
-    p->tcp_up--;
-    if (p->tcp_up != (PEER_TCP_MAGIC_COUNT - 1))
-	return;
-    p->last_fail_time = squid_curtime;
-    eventAdd("peerCheckConnect", peerCheckConnect, p, 30.0, 1);
-}
-
 static void
 peerCountMcastPeersSchedule(peer * p, time_t when)
 {
@@ -1073,7 +1134,7 @@ peerCountMcastPeersStart(void *data)
     mem->ircb_data = psstate;
     mcastSetTtl(theOutIcpConnection, p->mcast.ttl);
     p->mcast.id = mem->id;
-    reqnum = icpSetCacheKey(fake->key);
+    reqnum = icpSetCacheKey(fake->hash.key);
     query = icpCreateMessage(ICP_QUERY, 0, url, reqnum, 0);
     icpUdpSend(theOutIcpConnection,
 	&p->in_addr,
@@ -1236,9 +1297,9 @@ dump_peers(StoreEntry * sentry, peer * peers)
 #if USE_HTCP
 	}
 #endif
-	if (e->last_fail_time) {
+	if (e->stats.last_connect_failure) {
 	    storeAppendPrintf(sentry, "Last failed connect() at: %s\n",
-		mkhttpdlogtime(&(e->last_fail_time)));
+		mkhttpdlogtime(&(e->stats.last_connect_failure)));
 	}
 	if (e->peer_domain != NULL) {
 	    storeAppendPrintf(sentry, "DOMAIN LIST: ");
