@@ -1,19 +1,19 @@
 
 /*
- * $Id: structs.h,v 1.301.2.24 2000/01/10 06:02:12 wessels Exp $
+ * $Id: structs.h,v 1.361.2.13 2001/03/01 21:49:26 hno Exp $
  *
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  Duane Wessels and the University of California San Diego.  Please
- *  see the COPYRIGHT file for full details.  Squid incorporates
- *  software developed and/or copyrighted by other sources.  Please see
- *  the CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -49,12 +49,6 @@ struct _acl_ip_data {
     acl_ip_data *next;		/* used for parsing, not for storing */
 };
 
-struct _acl_snmp_comm {
-    char *name;
-    void *community;
-    acl_snmp_comm *next;
-};
-
 struct _acl_time_data {
     int weekbits;
     int start;
@@ -68,9 +62,7 @@ struct _acl_name_list {
 };
 
 struct _acl_proxy_auth_user {
-    /* first two items must be same as hash_link */
-    char *user;
-    acl_proxy_auth_user *next;
+    hash_link hash;		/* must be first */
     /* extra fields for proxy_auth */
     char *passwd;
     int passwd_ok;		/* 1 = passwd checked OK */
@@ -101,6 +93,11 @@ struct _String {
     char *buf;
 };
 
+struct _http_version_t {
+    unsigned int major;
+    unsigned int minor;
+};
+
 #if SQUID_SNMP
 
 struct _snmp_request_t {
@@ -159,11 +156,6 @@ struct _aclCheck_t {
     void *callback_data;
 };
 
-struct _aio_result_t {
-    int aio_return;
-    int aio_errno;
-};
-
 struct _wordlist {
     char *key;
     wordlist *next;
@@ -221,6 +213,11 @@ struct _delayConfig {
 
 #endif
 
+struct _RemovalPolicySettings {
+    char *type;
+    wordlist *args;
+};
+
 struct _SquidConfig {
     struct {
 	size_t maxSize;
@@ -238,14 +235,8 @@ struct _SquidConfig {
 	int pct;
 	size_t max;
     } quickAbort;
-#if HEAP_REPLACEMENT
-    char *replPolicy;
-#else
-    /* 
-     * Note: the non-LRU policies do not use referenceAge, but we cannot
-     * remove it until we find out how to implement #else for cf_parser.c
-     */
-#endif
+    RemovalPolicySettings *replPolicy;
+    RemovalPolicySettings *memPolicy;
     time_t referenceAge;
     time_t negativeTtl;
     time_t negativeDnsTtl;
@@ -265,6 +256,10 @@ struct _SquidConfig {
 	int mcast_icp_query;	/* msec */
 #if USE_IDENT
 	time_t ident;
+#endif
+#if !USE_DNSSERVERS
+	time_t idns_retransmit;
+	time_t idns_query;
 #endif
     } Timeout;
     size_t maxRequestHeaderSize;
@@ -293,6 +288,7 @@ struct _SquidConfig {
 	struct in_addr router;
 	struct in_addr incoming;
 	struct in_addr outgoing;
+	int version;
     } Wccp;
 #endif
     char *as_whois_server;
@@ -301,25 +297,42 @@ struct _SquidConfig {
 	char *access;
 	char *store;
 	char *swap;
+#if USE_USERAGENT_LOG
 	char *useragent;
+#endif
+#if USE_REFERER_LOG
+	char *referer;
+#endif
+#if WIP_FWD_LOG
+	char *forward;
+#endif
 	int rotateNumber;
     } Log;
     char *adminEmail;
     char *effectiveUser;
     char *effectiveGroup;
     struct {
+#if USE_DNSSERVERS
 	char *dnsserver;
+#endif
 	wordlist *redirect;
 	wordlist *authenticate;
+#if USE_ICMP
 	char *pinger;
+#endif
+#if USE_UNLINKD
 	char *unlinkd;
+#endif
     } Program;
+#if USE_DNSSERVERS
     int dnsChildren;
+#endif
     int redirectChildren;
     int authenticateChildren;
-    int authenticateTTL;
-    int authenticateIpTTL;
+    time_t authenticateTTL;
+    time_t authenticateIpTTL;
     struct {
+	int single_host;
 	char *host;
 	u_short port;
     } Accel;
@@ -365,11 +378,14 @@ struct _SquidConfig {
 	int size;
     } fqdncache;
     int minDirectHops;
+    int minDirectRtt;
     cachemgr_passwd *passwd_list;
     struct {
 	int objectsPerBucket;
 	size_t avgObjectSize;
 	size_t maxObjectSize;
+	size_t minObjectSize;
+	size_t maxInMemObjSize;
     } Store;
     struct {
 	int high;
@@ -378,8 +394,9 @@ struct _SquidConfig {
     } Netdb;
     struct {
 	int log_udp;
-	int enable_purge;
+#if USE_DNSSERVERS
 	int res_defnames;
+#endif
 	int anonymizer;
 	int client_db;
 	int query_icmp;
@@ -402,12 +419,19 @@ struct _SquidConfig {
 	int offline;
 	int redir_rewrites_host;
 	int prefer_direct;
+	int nonhierarchical_direct;
 	int strip_query_terms;
 	int redirector_bypass;
 	int ignore_unknown_nameservers;
+	int client_pconns;
+	int server_pconns;
 #if USE_CACHE_DIGESTS
 	int digest_generation;
 #endif
+	int log_ip_on_direct;
+	int authenticateIpTTLStrict;
+	int ie_refresh;
+	int pipeline_prefetch;
     } onoff;
     acl *aclList;
     struct {
@@ -433,6 +457,7 @@ struct _SquidConfig {
 	size_t list_width;
 	int list_wrap;
 	char *anon_user;
+	int passive;
     } Ftp;
     refresh_t *Refresh;
     struct _cacheSwap {
@@ -486,6 +511,13 @@ struct _SquidConfig {
 	int rebuild_chunk_percentage;
     } digest;
 #endif
+    wordlist *ext_methods;
+    struct {
+	int high_rptm;
+	int high_pf;
+	size_t high_memory;
+    } warnings;
+    char *store_dir_select_algorithm;
 };
 
 struct _SquidConfig2 {
@@ -493,6 +525,11 @@ struct _SquidConfig2 {
 	char *prefix;
 	int on;
     } Accel;
+    struct {
+	int enable_purge;
+    } onoff;
+    uid_t effectiveUserID;
+    gid_t effectiveGroupID;
 };
 
 struct _close_handler {
@@ -621,7 +658,7 @@ struct _Packer {
 /* http status line */
 struct _HttpStatusLine {
     /* public, read only */
-    float version;
+    http_version_t version;
     const char *reason;		/* points to a _constant_ string (default or supplied), never free()d */
     http_status status;
 };
@@ -646,6 +683,7 @@ struct _HttpHdrCc {
     int mask;
     int max_age;
     int s_maxage;
+    int max_stale;
 };
 
 /* http byte-range-spec */
@@ -755,6 +793,7 @@ struct _HttpStateData {
     StoreEntry *entry;
     request_t *request;
     char *reply_hdr;
+    size_t reply_hdr_size;
     int reply_hdr_state;
     peer *peer;			/* peer request made to */
     int eof;			/* reached end-of-object? */
@@ -807,7 +846,7 @@ struct _AccessLogEntry {
 	method_t method;
 	int code;
 	const char *content_type;
-	float version;
+	http_version_t version;
     } http;
     struct {
 	icp_opcode opcode;
@@ -832,6 +871,8 @@ struct _AccessLogEntry {
 struct _clientHttpRequest {
     ConnStateData *conn;
     request_t *request;		/* Parsed URL ... */
+    store_client *sc;		/* The store_client we're using */
+    store_client *old_sc;	/* ... for entry to be validated */
     char *uri;
     char *log_uri;
     struct {
@@ -846,9 +887,8 @@ struct _clientHttpRequest {
 #if USE_CACHE_DIGESTS
     const char *lookup_type;	/* temporary hack: storeGet() result: HIT/MISS/NONE */
 #endif
-    http_status http_code;
     struct timeval start;
-    float http_ver;
+    http_version_t http_ver;
     int redirect_state;
     aclCheck_t *acl_checklist;	/* need ptr back so we can unreg if needed */
     clientHttpRequest *next;
@@ -893,49 +933,6 @@ struct _ipcache_addrs {
     unsigned char badcount;
 };
 
-struct _ip_pending {
-    IPH *handler;
-    void *handlerData;
-    ip_pending *next;
-};
-
-struct _ipcache_entry {
-    /* first two items must be equivalent to hash_link */
-    char *name;
-    ipcache_entry *next;
-    time_t lastref;
-    time_t expires;
-    ipcache_addrs addrs;
-    ip_pending *pending_head;
-    char *error_message;
-    struct timeval request_time;
-    dlink_node lru;
-    u_char locks;
-    ipcache_status_t status:3;
-};
-
-struct _fqdn_pending {
-    FQDNH *handler;
-    void *handlerData;
-    fqdn_pending *next;
-};
-
-struct _fqdncache_entry {
-    /* first two items must be equivalent to hash_link */
-    char *name;
-    fqdncache_entry *next;
-    time_t lastref;
-    time_t expires;
-    unsigned char name_count;
-    char *names[FQDN_MAX_NAMES + 1];
-    fqdn_pending *pending_head;
-    char *error_message;
-    struct timeval request_time;
-    dlink_node lru;
-    unsigned char locks;
-    fqdncache_status_t status:3;
-};
-
 struct _domain_ping {
     char *domain;
     int do_ping;		/* boolean */
@@ -971,6 +968,8 @@ struct _DigestFetchState {
     PeerDigest *pd;
     StoreEntry *entry;
     StoreEntry *old_entry;
+    store_client *sc;
+    store_client *old_sc;
     request_t *request;
     int offset;
     int mask_offset;
@@ -1038,8 +1037,11 @@ struct _peer {
 	int ignored_replies;
 	int n_keepalives_sent;
 	int n_keepalives_recv;
+	time_t probe_start;
 	time_t last_query;
 	time_t last_reply;
+	time_t last_connect_failure;
+	time_t last_connect_probe;
 	int logged_state;	/* so we can print dead/revived msgs */
     } stats;
     struct {
@@ -1073,6 +1075,7 @@ struct _peer {
 #if DELAY_POOLS
 	unsigned int no_delay:1;
 #endif
+	unsigned int allow_miss:1;
     } options;
     int weight;
     struct {
@@ -1091,10 +1094,10 @@ struct _peer {
     char *digest_url;
 #endif
     int tcp_up;			/* 0 if a connect() fails */
-    time_t last_fail_time;
     struct in_addr addresses[10];
     int n_addresses;
     int rr_count;
+    int rr_lastcount;
     peer *next;
     int test_fd;
 #if USE_CARP
@@ -1109,23 +1112,20 @@ struct _peer {
 };
 
 struct _net_db_name {
-    char *name;
-    net_db_name *htbl_next;
+    hash_link hash;		/* must be first */
     net_db_name *next;
     netdbEntry *net_db_entry;
 };
 
 struct _net_db_peer {
-    char *peername;
+    const char *peername;
     double hops;
     double rtt;
     time_t expires;
 };
 
 struct _netdbEntry {
-    /* first two items must be equivalent to hash_link */
-    char *key;
-    netdbEntry *next;
+    hash_link hash;		/* must be first */
     char network[16];
     int pings_sent;
     int pings_recv;
@@ -1171,6 +1171,7 @@ struct _ps_state {
     aclCheck_t *acl_checklist;
 };
 
+#if USE_ICMP
 struct _pingerEchoData {
     struct in_addr to;
     unsigned char opcode;
@@ -1187,6 +1188,8 @@ struct _pingerReplyData {
     char payload[PINGER_PAYLOAD_SZ];
 };
 
+#endif
+
 struct _icp_common_t {
     unsigned char opcode;	/* opcode */
     unsigned char version;	/* version number */
@@ -1235,13 +1238,46 @@ struct _store_client {
 	unsigned int store_copying:1;
 	unsigned int copy_event_pending:1;
     } flags;
-    store_client *next;
 #if DELAY_POOLS
     delay_id delay_id;
 #endif
+    dlink_node node;
 };
 
 
+/* Removal policies */
+
+struct _RemovalPolicyNode {
+    void *data;
+};
+
+struct _RemovalPolicy {
+    char *_type;
+    void *_data;
+    void (*Free) (RemovalPolicy * policy);
+    void (*Add) (RemovalPolicy * policy, StoreEntry * entry, RemovalPolicyNode * node);
+    void (*Remove) (RemovalPolicy * policy, StoreEntry * entry, RemovalPolicyNode * node);
+    void (*Referenced) (RemovalPolicy * policy, const StoreEntry * entry, RemovalPolicyNode * node);
+    void (*Dereferenced) (RemovalPolicy * policy, const StoreEntry * entry, RemovalPolicyNode * node);
+    RemovalPolicyWalker *(*WalkInit) (RemovalPolicy * policy);
+    RemovalPurgeWalker *(*PurgeInit) (RemovalPolicy * policy, int max_scan);
+};
+
+struct _RemovalPolicyWalker {
+    RemovalPolicy *_policy;
+    void *_data;
+    const StoreEntry *(*Next) (RemovalPolicyWalker * walker);
+    void (*Done) (RemovalPolicyWalker * walker);
+};
+
+struct _RemovalPurgeWalker {
+    RemovalPolicy *_policy;
+    void *_data;
+    int scanned, max_scan, locked;
+    StoreEntry *(*Next) (RemovalPurgeWalker * walker);
+    void (*Done) (RemovalPurgeWalker * walker);
+};
+
 /* This structure can be freed while object is purged out from memory */
 struct _MemObject {
     method_t method;
@@ -1249,10 +1285,11 @@ struct _MemObject {
     mem_hdr data_hdr;
     off_t inmem_hi;
     off_t inmem_lo;
-    store_client *clients;
+    dlink_list clients;
     int nclients;
     struct {
 	off_t queue_offset;	/* relative to in-mem data */
+	mem_node *memnode;	/* which node we're currently paging out */
 	storeIOState *sio;
     } swapout;
     HttpReply *reply;
@@ -1266,37 +1303,28 @@ struct _MemObject {
 	void *data;
     } abort;
     char *log_url;
-#if HEAP_REPLACEMENT
-    /* 
-     * A MemObject knows where it is in the in-memory heap.
-     */
-    heap_node *node;
-#else
-    dlink_node lru;
-#endif
+    RemovalPolicyNode repl;
     int id;
     ssize_t object_sz;
     size_t swap_hdr_sz;
+#if URL_CHECKSUM_DEBUG
+    unsigned int chksum;
+#endif
 };
 
 struct _StoreEntry {
-    /* first two items must be same as hash_link */
-    const cache_key *key;
-    StoreEntry *next;
+    hash_link hash;		/* must be first */
     MemObject *mem_obj;
+    RemovalPolicyNode repl;
     time_t timestamp;
     time_t lastref;
     time_t expires;
     time_t lastmod;
     size_t swap_file_sz;
+    sfileno swap_filen:25;
+    sdirno swap_dirn:7;
     u_short refcount;
     u_short flags;
-    sfileno swap_file_number;
-#if HEAP_REPLACEMENT
-    heap_node *node;
-#else
-    dlink_node lru;
-#endif
     u_short lock_count;		/* Assume < 65536! */
     mem_status_t mem_status:3;
     ping_status_t ping_status:3;
@@ -1305,20 +1333,36 @@ struct _StoreEntry {
 };
 
 struct _SwapDir {
-    swapdir_t type;
-    fileMap *map;
+    char *type;
     int cur_size;
+    int low_size;
     int max_size;
     char *path;
     int index;			/* This entry's index into the swapDirs array */
     int suggest;
+    ssize_t max_objsize;
+    RemovalPolicy *repl;
+    int removals;
+    int scanned;
     struct {
 	unsigned int selected:1;
 	unsigned int read_only:1;
     } flags;
-    STINIT *init;
-    STNEWFS *newfs;
+    STINIT *init;		/* Initialise the fs */
+    STNEWFS *newfs;		/* Create a new fs */
+    STDUMP *dump;		/* Dump fs config snippet */
+    STFREE *freefs;		/* Free the fs data */
+    STDBLCHECK *dblcheck;	/* Double check the obj integrity */
+    STSTATFS *statfs;		/* Dump fs statistics */
+    STMAINTAINFS *maintainfs;	/* Replacement maintainence */
+    STCHECKOBJ *checkobj;	/* Check if the fs will store an object */
+    /* These two are notifications */
+    STREFOBJ *refobj;		/* Reference this object */
+    STUNREFOBJ *unrefobj;	/* Unreference this object */
+    STCALLBACK *callback;	/* Handle pending callbacks */
+    STSYNC *sync;		/* Sync the directory */
     struct {
+	STOBJCREATE *create;
 	STOBJOPEN *open;
 	STOBJCLOSE *close;
 	STOBJREAD *read;
@@ -1330,18 +1374,18 @@ struct _SwapDir {
 	STLOGCLOSE *close;
 	STLOGWRITE *write;
 	struct {
-	    STLOGCLEANOPEN *open;
+	    STLOGCLEANSTART *start;
+	    STLOGCLEANNEXTENTRY *nextentry;
 	    STLOGCLEANWRITE *write;
+	    STLOGCLEANDONE *done;
 	    void *state;
 	} clean;
+	int writes_since_clean;
     } log;
-    union {
-	struct {
-	    int l1;
-	    int l2;
-	    int swaplog_fd;
-	} ufs;
-    } u;
+    struct {
+	int blksize;
+    } fs;
+    void *fsdata;
 };
 
 struct _request_flags {
@@ -1371,10 +1415,13 @@ struct _link_list {
 };
 
 struct _storeIOState {
-    sfileno swap_file_number;
+    sdirno swap_dirn;
+    sfileno swap_filen;
+    StoreEntry *e;		/* Need this so the FS layers can play god */
     mode_t mode;
     size_t st_size;		/* do stat(2) after read open */
-    off_t offset;		/* current offset pointer */
+    off_t offset;		/* current on-disk offset pointer */
+    STFNCB *file_callback;	/* called on delayed sfileno assignments */
     STIOCB *callback;
     void *callback_data;
     struct {
@@ -1384,28 +1431,7 @@ struct _storeIOState {
     struct {
 	unsigned int closing:1;	/* debugging aid */
     } flags;
-    union {
-	struct {
-	    int fd;
-	    struct {
-		unsigned int close_request:1;
-		unsigned int reading:1;
-		unsigned int writing:1;
-	    } flags;
-	} ufs;
-	struct {
-	    int fd;
-	    struct {
-		unsigned int close_request:1;
-		unsigned int reading:1;
-		unsigned int writing:1;
-		unsigned int opening:1;
-	    } flags;
-	    const char *read_buf;
-	    link_list *pending_writes;
-	    link_list *pending_reads;
-	} aufs;
-    } type;
+    void *fsstate;
 };
 
 struct _request_t {
@@ -1421,7 +1447,7 @@ struct _request_t {
     request_flags flags;
     HttpHdrCc *cache_control;
     HttpHdrRange *range;
-    float http_ver;
+    http_version_t http_ver;
     time_t ims;
     int imslen;
     int max_forwards;
@@ -1521,6 +1547,8 @@ struct _StatCounters {
 	int clients;
 	int requests;
 	int hits;
+	int mem_hits;
+	int disk_hits;
 	int errors;
 	kb_t kbytes_in;
 	kb_t kbytes_out;
@@ -1618,8 +1646,12 @@ struct _StatCounters {
 	int selects;
 #endif
     } syscalls;
-    int swap_files_cleaned;
     int aborted_requests;
+    struct {
+	int files_cleaned;
+	int outs;
+	int ins;
+    } swap;
 };
 
 /* per header statistics */
@@ -1645,9 +1677,13 @@ struct _tlv {
     tlv *next;
 };
 
+/*
+ * Do we need to have the dirn in here? I don't think so, since we already
+ * know the dirn .. 
+ */
 struct _storeSwapLogData {
     char op;
-    sfileno swap_file_number;
+    sfileno swap_filen;
     time_t timestamp;
     time_t lastref;
     time_t expires;
@@ -1683,9 +1719,7 @@ struct _MemPool {
 };
 
 struct _ClientInfo {
-    /* first two items must be equivalent to hash_link */
-    char *key;
-    ClientInfo *next;
+    hash_link hash;		/* must be first */
     struct in_addr addr;
     struct {
 	int result_hist[LOG_TYPE_MAX];
@@ -1727,6 +1761,9 @@ struct _FwdState {
     ErrorState *err;
     time_t start;
     int n_tries;
+#if WIP_FWD_LOG
+    http_status last_status;
+#endif
     struct {
 	unsigned int dont_retry:1;
 	unsigned int ftp_pasv_failed:1;
@@ -1816,3 +1853,75 @@ struct _store_rebuild_data {
     int bad_log_op;
     int zero_object_sz;
 };
+
+struct _PumpStateData {
+    FwdState *fwd;
+    request_t *req;
+    store_client *sc;		/* The store client we're using */
+    int c_fd;			/* client fd */
+    int s_fd;			/* server end */
+    int rcvd;			/* bytes received from client */
+    int sent;			/* bytes sent to server */
+    StoreEntry *request_entry;	/* the request entry */
+    StoreEntry *reply_entry;	/* the reply entry */
+    CWCB *callback;		/* what to do when we finish sending */
+    void *cbdata;		/* callback data passed to callback func */
+    struct {
+	unsigned int closing:1;
+    } flags;
+    struct _PumpStateData *next;
+};
+
+/*
+ * This defines an fs type
+ */
+
+struct _storefs_entry {
+    char *typestr;
+    STFSPARSE *parsefunc;
+    STFSRECONFIGURE *reconfigurefunc;
+    STFSSHUTDOWN *donefunc;
+};
+
+/*
+ * This defines an repl type
+ */
+
+struct _storerepl_entry {
+    char *typestr;
+    REMOVALPOLICYCREATE *create;
+};
+
+/*
+ * Async disk IO - this defines a async disk io queue
+ */
+
+struct _diskd_queue {
+    int smsgid;			/* send sysvmsg id */
+    int rmsgid;			/* recv sysvmsg id */
+    int wfd;			/* queue file descriptor ? */
+    int away;			/* number of requests away */
+    int sent_count;		/* number of messages sent */
+    int recv_count;		/* number of messages received */
+    struct {
+	char *buf;		/* shm buffer */
+	link_list *stack;
+	int id;			/* sysvshm id */
+    } shm;
+};
+
+struct _Logfile {
+    int fd;
+    char path[MAXPATHLEN];
+    char *buf;
+    size_t bufsz;
+    off_t offset;
+    struct {
+	unsigned int fatal:1;
+    } flags;
+};
+
+struct cache_dir_option {
+    char *name;
+    void (*parse) (SwapDir * sd, const char *option, const char *value, int reconfiguring);
+};
