@@ -1,21 +1,21 @@
 
 /*
- * $Id: tools.c,v 1.183.2.10 2000/01/05 23:45:34 wessels Exp $
+ * $Id: tools.c,v 1.197.2.4 2001/02/15 11:13:54 adrian Exp $
  *
  * DEBUG: section 21    Misc Functions
  * AUTHOR: Harvest Derived
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  Duane Wessels and the University of California San Diego.  Please
- *  see the COPYRIGHT file for full details.  Squid incorporates
- *  software developed and/or copyrighted by other sources.  Please see
- *  the CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ The Squid Cache (version %s) died.\n\
 You've encountered a fatal error in the Squid Cache version %s.\n\
 If a core file was created (possibly in the swap directory),\n\
 please execute 'gdb squid core' or 'dbx squid core', then type 'where',\n\
-and report the trace back to squid-bugs@ircache.net.\n\
+and report the trace back to squid-bugs@squid-cache.org.\n\
 \n\
 Thanks!\n"
 
@@ -53,6 +53,13 @@ extern void log_trace_done();
 extern void log_trace_init(char *);
 #endif
 
+#ifdef _SQUID_LINUX_
+/* Workaround for crappy glic header files */
+extern int backtrace(void *, int);
+extern void backtrace_symbols_fd(void *, int, int);
+extern int setresuid(uid_t, uid_t, uid_t);
+#endif /* _SQUID_LINUX */
+
 extern void (*failure_notify) (const char *);
 
 void
@@ -82,12 +89,21 @@ static void
 mail_warranty(void)
 {
     FILE *fp = NULL;
-    char *filename;
     static char command[256];
+#if HAVE_MKSTEMP
+    char filename[] = "/tmp/squid-XXXXXX";
+    int tfd = mkstemp(filename);
+    if (tfd < 0)
+	return;
+    if ((fp = fdopen(tfd, "w")) == NULL)
+	return;
+#else
+    char *filename;
     if ((filename = tempnam(NULL, appname)) == NULL)
 	return;
     if ((fp = fopen(filename, "w")) == NULL)
 	return;
+#endif
     fprintf(fp, "From: %s\n", appname);
     fprintf(fp, "To: %s\n", Config.adminEmail);
     fprintf(fp, "Subject: %s\n", dead_msg());
@@ -254,6 +270,14 @@ death(int sig)
 	fflush(stdout);
     }
 #endif /* _SQUID_SOLARIS_ */
+#if HAVE_BACKTRACE_SYMBOLS_FD
+    {
+	static void *(callarray[8192]);
+	int n;
+	n = backtrace(callarray, 8192);
+	backtrace_symbols_fd(callarray, n, fileno(debug_log));
+    }
+#endif
 #endif /* PRINT_STACK_TRACE */
 
 #if SA_RESETHAND == 0
@@ -416,46 +440,47 @@ getMyHostname(void)
     const struct hostent *h = NULL;
     if (Config.visibleHostname != NULL)
 	return Config.visibleHostname;
-    if (present) {
-	(void) 0;
-    } else if (Config.Sockaddr.http->s.sin_addr.s_addr != any_addr.s_addr) {
+    if (present)
+	return host;
+    host[0] = '\0';
+    if (Config.Sockaddr.http->s.sin_addr.s_addr != any_addr.s_addr) {
 	/*
 	 * If the first http_port address has a specific address, try a
 	 * reverse DNS lookup on it.
 	 */
-	host[0] = '\0';
 	h = gethostbyaddr((char *) &Config.Sockaddr.http->s.sin_addr,
 	    sizeof(Config.Sockaddr.http->s.sin_addr), AF_INET);
 	if (h != NULL) {
 	    /* DNS lookup successful */
 	    /* use the official name from DNS lookup */
-	    strcpy(host, h->h_name);
-	    debug(50, 4) ("getMyHostname: resolved tcp_incoming_addr to '%s'\n",
+	    xstrncpy(host, h->h_name, SQUIDHOSTNAMELEN);
+	    debug(50, 4) ("getMyHostname: resolved %s to '%s'\n",
+		inet_ntoa(Config.Sockaddr.http->s.sin_addr),
 		host);
 	    present = 1;
-	} else {
-	    debug(50, 6) ("getMyHostname: failed to resolve tcp_incoming_addr\n");
+	    return host;
 	}
-    } else {
-	/*
-	 * Get the host name and store it in host to return
-	 */
-	host[0] = '\0';
-	if (gethostname(host, SQUIDHOSTNAMELEN) == -1) {
-	    debug(50, 1) ("getMyHostname: gethostname failed: %s\n",
-		xstrerror());
-	} else {
-	    if ((h = gethostbyname(host)) != NULL) {
-		debug(50, 6) ("getMyHostname: '%s' resolved into '%s'\n",
-		    host, h->h_name);
-		/* DNS lookup successful */
-		/* use the official name from DNS lookup */
-		strcpy(host, h->h_name);
-	    }
-	}
-	present = 1;
+	debug(50, 1) ("WARNING: failed to resolve %s to a hostname\n",
+	    inet_ntoa(Config.Sockaddr.http->s.sin_addr));
     }
-    return present ? host : NULL;
+    /*
+     * Get the host name and store it in host to return
+     */
+    if (gethostname(host, SQUIDHOSTNAMELEN) < 0) {
+	debug(50, 1) ("WARNING: gethostname failed: %s\n", xstrerror());
+    } else if ((h = gethostbyname(host)) == NULL) {
+	debug(50, 1) ("WARNING: gethostbyname failed for %s\n", host);
+    } else {
+	debug(50, 6) ("getMyHostname: '%s' resolved into '%s'\n",
+	    host, h->h_name);
+	/* DNS lookup successful */
+	/* use the official name from DNS lookup */
+	xstrncpy(host, h->h_name, SQUIDHOSTNAMELEN);
+	present = 1;
+	return host;
+    }
+    fatal("Could not determine fully qualified hostname.  Please set 'visible_hostname'\n");
+    return NULL;		/* keep compiler happy */
 }
 
 const char *
@@ -467,7 +492,7 @@ uniqueHostname(void)
 void
 safeunlink(const char *s, int quiet)
 {
-    Counter.syscalls.disk.unlinks++;
+    statCounter.syscalls.disk.unlinks++;
     if (unlink(s) < 0 && !quiet)
 	debug(50, 1) ("safeunlink: Couldn't delete %s: %s\n", s, xstrerror());
 }
@@ -480,38 +505,28 @@ safeunlink(const char *s, int quiet)
 void
 leave_suid(void)
 {
-    struct passwd *pwd = NULL;
-    struct group *grp = NULL;
-    gid_t gid;
     debug(21, 3) ("leave_suid: PID %d called\n", getpid());
     if (geteuid() != 0)
 	return;
     /* Started as a root, check suid option */
     if (Config.effectiveUser == NULL)
 	return;
-    if ((pwd = getpwnam(Config.effectiveUser)) == NULL)
-	return;
-    if (Config.effectiveGroup && (grp = getgrnam(Config.effectiveGroup))) {
-	gid = grp->gr_gid;
-    } else {
-	gid = pwd->pw_gid;
-    }
 #if HAVE_SETGROUPS
-    setgroups(1, &gid);
+    setgroups(1, &Config2.effectiveGroupID);
 #endif
-    if (setgid(gid) < 0)
-	debug(50, 1) ("leave_suid: setgid: %s\n", xstrerror());
+    if (setgid(Config2.effectiveGroupID) < 0)
+	debug(50, 0) ("ALERT: setgid: %s\n", xstrerror());
     debug(21, 3) ("leave_suid: PID %d giving up root, becoming '%s'\n",
-	getpid(), pwd->pw_name);
+	getpid(), Config.effectiveUser);
 #if HAVE_SETRESUID
-    if (setresuid(pwd->pw_uid, pwd->pw_uid, 0) < 0)
-	debug(50, 1) ("leave_suid: setresuid: %s\n", xstrerror());
+    if (setresuid(Config2.effectiveUserID, Config2.effectiveUserID, 0) < 0)
+	debug(50, 0) ("ALERT: setresuid: %s\n", xstrerror());
 #elif HAVE_SETEUID
-    if (seteuid(pwd->pw_uid) < 0)
-	debug(50, 1) ("leave_suid: seteuid: %s\n", xstrerror());
+    if (seteuid(Config2.effectiveUserID) < 0)
+	debug(50, 0) ("ALERT: seteuid: %s\n", xstrerror());
 #else
-    if (setuid(pwd->pw_uid) < 0)
-	debug(50, 1) ("leave_suid: setuid: %s\n", xstrerror());
+    if (setuid(Config2.effectiveUserID) < 0)
+	debug(50, 0) ("ALERT: setuid: %s\n", xstrerror());
 #endif
 }
 
@@ -560,7 +575,7 @@ writePidFile(void)
 	return;
     enter_suid();
     old_umask = umask(022);
-    fd = file_open(f, O_WRONLY | O_CREAT | O_TRUNC);
+    fd = file_open(f, O_WRONLY | O_CREAT | O_TRUNC | O_TEXT);
     umask(old_umask);
     leave_suid();
     if (fd < 0) {
@@ -736,8 +751,6 @@ logsFlush(void)
 {
     if (debug_log)
 	fflush(debug_log);
-    if (cache_useragent_log)
-	fflush(cache_useragent_log);
 }
 
 char *
@@ -848,13 +861,13 @@ debugObj(int section, int level, const char *label, void *obj, ObjPackMethod pm)
 int
 stringHasWhitespace(const char *s)
 {
-    return (strcspn(s, w_space) != strlen(s));
+    return strpbrk(s, w_space) != NULL;
 }
 
 void
 linklistPush(link_list ** L, void *p)
 {
-    link_list *l = xmalloc(sizeof(*l));
+    link_list *l = memAllocate(MEM_LINK_LIST);
     l->next = NULL;
     l->ptr = p;
     while (*L)
@@ -872,11 +885,10 @@ linklistShift(link_list ** L)
     l = *L;
     p = l->ptr;
     *L = (*L)->next;
-    xfree(l);
+    memFree(l, MEM_LINK_LIST);
     return p;
 }
 
-
 /*
  * Same as rename(2) but complains if something goes wrong;
  * the caller is responsible for handing and explaining the 
@@ -905,3 +917,17 @@ stringHasCntl(const char *s)
     }
     return 0;
 }
+
+/*
+ * isPowTen returns true if its argument is an integer power of
+ * 10.  Its used for logging of certain error messages that can
+ * occur often, but that we don't want to fill cache.log with.
+ */
+int
+isPowTen(int count)
+{
+    double x = log(count) / log(10.0);
+    if (0.0 != x - (double) (int) x)
+	return 0;
+    return 1;
+}
